/*==========================================================================
**
**	psdb_fio.c
**
**	PSDB IO library functions for Linux 
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	ram	01/12/99 v1.0	Created
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/

#include <stdio.h>
#include <pdk_port.h>
#undef _KERNEL
#include <pdk_global.h>
#include <pdk_types.h>

#include <sys/stat.h>

#include <pdk_psdb.h>

#include <psdb_io.h>
#include <rmapi.h>

#include <sys/stat.h>
#include <sys/fcntl.h>
//#include <sys/sockio.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/if_ether.h>

#include <pdk_port.h>
#include <pdk_global.h>
#include <pdk_types.h>
#include <pio.h>
#include <pdk_psdb.h>

#define IFR_NAME "zrm"

#define MAX_DATA 1024 /* XXX link this to RMAPI structs */
typedef struct pio_fdesc_s {
	struct ifreq	pio_ifr; 
	int		pio_sd;	/* socket descriptor */
	/* the first 32 bits are a stream handle generated by the driver */
	uint32_t	pio_stream_handle;
	char		pio_data[MAX_DATA];
} pio_fdesc_t;

#define MAX_FDS NUM_PORTS	/* arbitrary */
pio_fdesc_t fds_array[MAX_FDS]={0};

extern int32_t rmapi_debug;

void pio_write(rmapi_fd_t psdb_fd, pdk_psdb_rhdr_t *psdb_data, int32_t psdb_size);
pio_fdesc_t *pio_getnext_fd(void);
void pio_atexit();

#define DEBUG1 if (rmapi_debug>0) printf
#define DEBUG2 if (rmapi_debug>1) printf
#define DEBUG3 if (rmapi_debug>2) printf
#define DEBUG4 if (rmapi_debug>3) printf

static int first_open = 1;
uint32_t num_open=0;

rmapi_fd_t
psdb_fio_open(dev, flags)
char *dev;
uint32_t flags;
{
	pio_fdesc_t *fds;
	int i;

	if (first_open) {
		for (i=0;i< MAX_FDS;i++) {
			fds = &fds_array[i];
			fds->pio_sd = -1;
		}

		first_open = 0;
		atexit(pio_atexit);
	}

	fds = pio_getnext_fd();

	if (fds == 0)
		err(1,"file descriptor");

	fds->pio_sd = socket(AF_INET, SOCK_DGRAM, 0);
	if (fds->pio_sd < 0) 
		err(1,"socket");

	strncpy(fds->pio_ifr.ifr_name, IFR_NAME, sizeof(IFR_NAME));
	fds->pio_stream_handle=0;
	fds->pio_ifr.ifr_data = (caddr_t)&fds->pio_stream_handle;
	if (flags) {
		if (ioctl(fds->pio_sd, SIOZOPEN, (caddr_t)&fds->pio_ifr))
			err(1,"SIOZOPEN");
	}
	else {
		if (ioctl(fds->pio_sd, SIOZBLOPEN, (caddr_t)&fds->pio_ifr))
			err(1,"SIOZBLOPEN");
	}

	DEBUG2("psdb_fio_open: stream handle %x opened\n",fds->pio_stream_handle);
	if (fds->pio_stream_handle == -1)
		return((rmapi_fd_t) -1);

	return((rmapi_fd_t) fds);
}

uint32_t
psdb_fio_close(fd)
rmapi_fd_t fd;
{
	pio_fdesc_t *fds;

	fds = (pio_fdesc_t *) fd;

	DEBUG2("psdb_fio_close: stream handle %x closed\n",fds->pio_stream_handle);
	if (fds->pio_stream_handle == -1)
		return(0);

	fds->pio_ifr.ifr_data = (caddr_t)&fds->pio_stream_handle;
	if (ioctl(fds->pio_sd, SIOZCLOSE, (caddr_t)&fds->pio_ifr))
		err(1,"SIOZCLOSE");

	close(fds->pio_sd);
	fds->pio_sd = -1;
	fds->pio_stream_handle = 0;
	num_open--;
}

uint32_t
psdb_fio_send(fd, prp, read_flag)
rmapi_fd_t fd;
pdk_psdb_rhdr_t *prp;
int32_t read_flag;
{
	pdk_psdb_error_t pe;
	uint32_t dev_id;
	int32_t size;
	int32_t read_size;

	size = psdb_pton(prp->ppr_header.Size);
	dev_id = psdb_pton(prp->ppr_dev_id);

	DEBUG3("psdb_fio_send(%d): send record type 0x%x of %d bytes\n", dev_id, psdb_pton(prp->ppr_header.Type), size);

	pio_write(fd, prp, size); 

	read_size = pio_read(fd, (pdk_psdb_rhdr_t *)&pe, sizeof(pe));

	DEBUG3("psdb_fio_send(%d): responce type 0x%x received size %d\n", dev_id, psdb_pton(pe.pe_header.ppr_header.Type), size);

	if (read_size != sizeof(pe))
		return(PSDB_PROTOCOL_ERROR);

	if (pe.pe_header.ppr_header.Type == psdb_ntop(PSDB_TYPE_ERROR)) {
		DEBUG3("psdb_fio_send(%d): Error responce %d\n", dev_id, psdb_pton(pe.pe_error));
		if (pe.pe_error_type != prp->ppr_header.Type)
			return(PSDB_PROTOCOL_ERROR);

		return(psdb_pton(pe.pe_error));
	}
	DEBUG3("psdb_fio_send(%d): No error\n", dev_id);

	if (read_flag) {
		read_size = pio_read(fd, prp, size);
		if (read_size != size) {
			return(PSDB_PROTOCOL_ERROR);
		}
	}

	return(0);
}

uint32_t
psdb_fio_get(fd, ptr, len)
rmapi_fd_t fd;
void *ptr;
uint32_t len;
{
	return(pio_read(fd, ptr, len));
}

pio_fdesc_t *
pio_getnext_fd(void)
{
	int i;
	pio_fdesc_t *fds;

	if (num_open > MAX_FDS)
		return 0;

	for (i=0; i<MAX_FDS; i++) {
		fds=&fds_array[i];
		if (fds->pio_sd == -1) {
			num_open++;
			return(fds);
		}
	}

	return((pio_fdesc_t *)0);
}

void
pio_atexit()
{
	pio_fdesc_t *fds;
	int i;

	for (i=0; i<MAX_FDS;i++) {
		fds = &fds_array[i];
		if (fds->pio_sd != -1)
			psdb_fio_close((rmapi_fd_t) fds);
	}
}

int32_t 
pio_read(psdb_fd, psdb_data, psdb_size)
rmapi_fd_t psdb_fd;
pdk_psdb_rhdr_t *psdb_data;
int32_t psdb_size;
{
	int32_t size;
	pdk_psdb_error_t pe;
	pdk_psdb_rhdr_t *psdb;
	pio_fdesc_t *fds;

	fds = (pio_fdesc_t *) psdb_fd;

	/* have to do this copy to get size and type in header */
	psdb_data->ppr_header.Size = psdb_pton(psdb_size);

	memcpy(fds->pio_data, psdb_data, psdb_size);
	fds->pio_ifr.ifr_data = (caddr_t)&fds->pio_stream_handle;

	if (ioctl(fds->pio_sd, SIOZREAD, (caddr_t)&fds->pio_ifr))
		err(1,"SIOZREAD");

	memcpy(psdb_data, fds->pio_data, psdb_size);
	psdb = (pdk_psdb_rhdr_t *) fds->pio_data;
	size = psdb_pton(psdb->ppr_header.Size);
	DEBUG3("pio_read size: %d\n",size);

	return(size);
}


void 
pio_write(psdb_fd, psdb_data, psdb_size)
rmapi_fd_t psdb_fd;
pdk_psdb_rhdr_t *psdb_data;
int32_t psdb_size;
{
	int32_t size;
	pio_fdesc_t *fds;
	pdk_psdb_rhdr_t *psdb;
	pdk_psdb_rhdr_t *pcsdb_data;

	fds = (pio_fdesc_t *)psdb_fd;
	psdb = psdb_data;
	pcsdb_data = (pdk_psdb_rhdr_t *) fds->pio_data;

	size = psdb_pton(psdb->ppr_header.Size);
	DEBUG3("pio_write size: %d %d\n",size,psdb_size);
	memcpy(fds->pio_data, psdb_data, psdb_size);
	fds->pio_ifr.ifr_data = (caddr_t)&fds->pio_stream_handle;

	if (ioctl(fds->pio_sd, SIOZWRITE, (caddr_t)&fds->pio_ifr))
		err(1,"SIOZWRITE");
}

