%{
#include <stdio.h>
#include <pdk_port.h>
#include <pdk_global.h>
#include <pdk_types.h>
#include <pdk_errno.h>
#include <pdk_psdb.h>

#include <rlhtml.h>
extern int lineno;

uint32_t rlhtml_opcode;
int32_t rlhtml_input_index = 0;
int32_t rlhtml_dev_id;
rlhtml_input_pair_t rlhtml_rip_array[RLHTML_INPUT_FIELDS];
rlhtml_output_format_t rlhtml_rof;

extern int32_t rl_debug;

#define DEBUG1 if (rl_debug>0) printf
#define DEBUG2 if (rl_debug>1) printf
#define DEBUG3 if (rl_debug>2) printf
#define DEBUG4 if (rl_debug>3) printf

%}
%union {
	int32_t dig;
	char chr[2];
}

%token END
%token <dig> DIGIT
%token DOLLAR
%token COMMA
%token ZNYX
%token HTML_START
%token <chr> CHR
%%
/* ($ZF$OP$F$F$F$$F,M) where
	$ZF	Where F is a number representing the output format.
	$OP	Opcode given to rmapi_ctl.
	$DE	Dev/PPA.  The 3rd long word of the rmapi_ctl data block.
	$F,V$F,V ...	1 or more field value pair.  The 32-bit
			value is set in the specified field of the
			parameter passed to rmapi_ctl.  The size is calculated
			All unspecified fields are set zero.
	$$
	F,M ...	field format pair.  The field is taken from the returned result
		of the rmapi_ctl and displayed according to the given format M.
		M=1 raw data (%d)
		M=2 ascii interpretation e.g. UP, DOWN etc. (%s)
		M=3 ascii color interpretation e.g. RED, YELLOW, GREEN etc. (%s)
*/
statements:	statement statements
	|	statement
	;

statement: znyx opcode devid input_pairs output_field end
	|  html
	|  char
		;
html: HTML_START 
		{
			DEBUG3("HTML_START\n");
		}
	;
znyx: ZNYX DIGIT
		{
			rlhtml_rof.rof_format = $2;
			DEBUG3("znyx: \n");
		}
	;

devid: DOLLAR DIGIT
		{
			rlhtml_dev_id=$2;
			DEBUG3("dev_id: dev_id %d\n",$2);
		}

opcode:	DOLLAR DIGIT
	{
		rlhtml_opcode = $2;
		DEBUG3("opcode: 0x%x\n",$2);
	}
	;
input_pairs:	DOLLAR DOLLAR
	|	DOLLAR input_pair
	|	DOLLAR input_pair input_pairs
	;

input_pair:	DIGIT COMMA DIGIT
		{
			rlhtml_rip_array[rlhtml_input_index].rip_field = $1;
			rlhtml_rip_array[rlhtml_input_index++].rip_value = $3;
			DEBUG3("input_pair: %d, %d\n",$1, $3);
		}
	;

output_field:	DIGIT
		{
			rlhtml_rof.rof_field = $1;
			DEBUG3("output_field: %d\n",$1);
		}
	;

end : DOLLAR END 
		{
			rlhtml_args_t erlha={0};
			int32_t i;

			DEBUG3("END\n");
			erlha.ra_opcode = rlhtml_opcode;
			erlha.ra_num_input_pairs = rlhtml_input_index;
			erlha.ra_dev_id = rlhtml_dev_id;
		 	memcpy(erlha.ra_input_pairs, rlhtml_rip_array, rlhtml_input_index*sizeof(rlhtml_input_pair_t));
			erlha.ra_output_format = rlhtml_rof;/* Structure copy */
			rlhtml_input_index = 0;
			if(rlhtml_process_opcode(&erlha)==0){
			} else {
				/* error */
			}
		}
	;
char:  CHR
		{
			DEBUG4("%c",$1);
		}
	;

%%

