/*==========================================================================
**
**	RLUTIL rlstats RAINlink statistics application
**
**==========================================================================
**
**	(c) Copyright 1999 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	tmh	11/05/99 v1.0	Created
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/

#include <stdio.h>
#include <pdk_port.h>
#undef _KERNEL
#include <pdk_global.h>
#include <pdk_types.h>
#include <pdk_errno.h>
#include <pdk_queue.h>
#include <rmapi.h>

int32_t rl_debug = 0;

#define DEBUG1 if (rl_debug>0) printf
#define DEBUG2 if (rl_debug>1) printf
#define DEBUG3 if (rl_debug>2) printf
#define DEBUG4 if (rl_debug>3) printf


int32_t rmapi_ppa_to_dev(uint32_t ppa, int rmapi_config);
void rc_show();
void rc_port_state(int32_t ppa);
void rc_port_speed(int32_t ppa);
void rc_speedometer_set_params(int32_t ppa);
void rc_pal_hw_info(int32_t ppa, int32_t fmt);

extern char *optarg;
extern int optind;
int lineno;
int rmapi_fd;
extern int rmapi_debug;
FILE *flist = 0;
int poll_rate=10000; /* in ms */
main(argc, argv)
int	argc;
char	*argv[];
{
	int c;
	int format = 0;
	int32_t ppa;
	int run_show = 0;
	int run_speedometer_show = 0;
	int run_speedometer_set = 0;
	int run_port_state = 0;
	int run_pal_hw_info = 0;
	char hostname_space[128];
	char *hostname;
	int32_t fmt=0;

	hostname = 0;

	if (argc < 2) {
		usage("");
		return(-1);
	}

	while ((c = getopt(argc, argv, "h:o:m:i:s:r:d:xz")) != -1) {
		switch (c) {
		case 'h':
			strncpy(hostname_space, optarg, 128);
			hostname = hostname_space;
			break;

		case 'd':
			rl_debug = atoi(optarg);
			rmapi_debug = rl_debug;
			break;

		case 'z':
			run_show = 1;
			break;

		case 'm':
			if (optind > argc) {
				usage("Must specify a ppa");
				exit(1);
			}

			if (isdigit(*optarg) == 0) {
				usage("Must specify ppa as a number");
				exit(0);
			}

			ppa = atoi(optarg);

			run_port_state = 1;

			break;

		case 'i':
			if (optind > argc) {
				usage("Must specify a ppa");
				exit(1);
			}

			if (isdigit(*optarg) == 0) {
				usage("Must specify ppa as a number");
				exit(0);
			}

			ppa = atoi(optarg);

			run_pal_hw_info = 1;

			break;

		case 'x':
			fmt = 1;
			break;

		case 'o':
			if (optind > argc) {
				usage("Must specify a ppa");
				exit(1);
			}

			if (isdigit(*optarg) == 0) {
				usage("Must specify ppa as a number");
				exit(0);
			}

			ppa = atoi(optarg);
			run_speedometer_show = 1;
			break;


		case 's':
			if (optind > argc) {
				usage("Must specify a ppa");
				exit(1);
			}

			if (isdigit(*optarg) == 0) {
				usage("Must specify ppa as a number");
				exit(0);
			}

			ppa = atoi(optarg);

			run_speedometer_set = 1;
			break;
		
		case 'r':
			if (optind > argc) {
				usage("Must specify a polling rate");
				exit(1);
			}

			if (isdigit(*optarg) == 0) {
				usage("Must specify polling rate as a number");
				exit(0);
			}

			poll_rate = atoi(optarg);

			run_speedometer_set = 1;
			break;

		default:
			usage("");
			return(-1);
		}
	}
	if ((rmapi_fd = rmapi_open(hostname, 0)) == -1) {
		DEBUG1("rmapi_open failed\n");
		exit(1);
	}

	if (run_show)
		rc_show();

	else if (ppa >= PSDB_MAX_PORTS)
		printf("Selected port %d out of range\n",ppa);

	else if (run_port_state)
		rc_port_state(ppa);

	else if (run_pal_hw_info)
		rc_pal_hw_info(ppa,fmt);

	else if (run_speedometer_set)
		rc_speedometer_set_params(ppa);

	else if (run_speedometer_show)
		rc_port_speed(ppa);

	rmapi_close(rmapi_fd);

}

rmapi_ppa_to_dev(ppa, rmapi_config)
uint32_t ppa;
int rmapi_config;
{
	rmapi_configuration_t rc={0};
	rlk_configuration_t *rcp;
	uint32_t dev_id;

	for (dev_id=0;dev_id<PSDB_MAX_PORTS;dev_id++) {
		rc.rc_ppa = -1;
		rc.rc_dev_id = dev_id;
		rc.rc_type = rmapi_config;
		rc.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) != 0)
			continue;

		rcp = &rc.rc_configuration;

		if (rcp->rc_version != RC_VERSION) {
			fprintf(stderr, "Version(%d) mismatch with RLK layer version(%d) in driver.  Need updated rlstats program\n", RC_VERSION, rcp->rc_version);
			return;
		}

		if (rmapi_config == RMAPI_CONFIG_FAILOVER_TYPE) {
			if ((rcp->rc_num_primary_ports > 1) &&
				(rcp->rc_failover_ppa == rcp->rc_ppa) &&
				(rcp->rc_ppa == ppa)) {

				return(dev_id);
			}
		}
		else {
			if ((rcp->rc_num_configured_ports > 1) &&
				(rcp->rc_configured_ports[0] == ppa)) {

				return(dev_id);
			}
		}
	}
	return(-1);
}

void
rc_show()
{
	rmapi_configuration_t rc={0}, rc2={0};
	rlk_configuration_t *rcp, *rcp2;
	int32_t ppa;
	int32_t mode;
	uint32_t dev_id;
	int j;
	int no_rainlink = 1;

	for (dev_id=0;dev_id<PSDB_MAX_PORTS;dev_id++) {
		rc.rc_ppa = -1;
		rc.rc_dev_id = dev_id;
		rc.rc_type = RMAPI_CONFIG_TRUNK_TYPE;
		rc.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) != 0)
			continue;


		rcp = &rc.rc_configuration;
		no_rainlink = 0;

		if (rcp->rc_version != RC_VERSION) {
			fprintf(stderr, "Version(%d) mismatch with RLK layer version(%d) in driver.  Need updated rlconfig program\n", RC_VERSION, rcp->rc_version);
			return;
		}

		/* Look for trunks first */
		if (rcp->rc_num_configured_ports < 2)
			continue;

		printf("trunk%d", dev_id);
		if (rcp->rc_mode != RLK_BALANCE_MODE) {
			printf("<");
			if (rcp->rc_mode & RLK_ROUND_ROBIN_MODE)
				printf("ROUND_ROBIN");
			else if (rcp->rc_mode & RLK_IP_TRUNKING_MODE)
				printf("IP_TRUNKING");
			else if (rcp->rc_mode & RLK_LAYER_2_MODE)
				printf("LAYER 2");
			else if (rcp->rc_mode & RLK_LAYER_3_MODE)
				printf("LAYER 3");

			if ((rcp->rc_mode & RLK_BALANCE_MODE) == 0) {
				if (rcp->rc_mode & ~RLK_BALANCE_MODE)
					printf(", ");

				printf("BALANCE_OFF");
			}

			printf(">");
		}
		ppa = rcp->rc_configured_ports[0];
		printf(" = znb%d", ppa);

		rc2.rc_ppa = ppa;
		rc2.rc_dev_id = -1;
		rc2.rc_type = 0;
		rc2.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc2) != 0) {
			fprintf(stderr, "RMAPI1 protocol error reading configuration for ppa %d\n", ppa);
			return;
		}

		rcp2 = &rc2.rc_configuration;

		if (rcp2->rc_linkup)
			printf("<UP>");
		else
			printf("<DOWN>");

		for (j=1;j<rcp->rc_num_configured_ports;j++) {
			ppa = rcp->rc_configured_ports[j];
			rc2.rc_ppa = ppa;
			rc2.rc_dev_id = -1;
			rc2.rc_type = 0;
			rc2.rc_configuration.rc_offset_ports = 0;
			if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc2) != 0) {
				fprintf(stderr, "RMAPI2 protocol error reading configuration for ppa %d\n", ppa);
				return;
			}
			rcp2 = &rc2.rc_configuration;

			printf(", znb%d", ppa);
			if (rcp2->rc_linkup)
				printf("<UP>");
			else
				printf("<DOWN>");
		}

		/* If not in a fail over group then it is an available device */
		if (rcp->rc_num_primary_ports < 2)
			printf(" ==> zrl%d", dev_id);

		printf("\n");
	}

	/* look for failover groups */
	for (dev_id=0;dev_id<PSDB_MAX_PORTS;dev_id++) {
		rc.rc_ppa = -1;
		rc.rc_dev_id = dev_id;
		rc.rc_type = RMAPI_CONFIG_FAILOVER_TYPE;
		rc.rc_configuration.rc_offset_ports = 0;
		if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc) != 0)
			continue;

		rcp = &rc.rc_configuration;
		no_rainlink = 0;

		if (rcp->rc_version != RC_VERSION) {
			fprintf(stderr, "Version(%d) mismatch with RLK layer version(%d) in driver.  Need updated rlconfig program\n", RC_VERSION, rcp->rc_version);
			return;
		}

		if (rcp->rc_num_primary_ports < 2)
			continue;

		/* Is this the assigned port for the failover group */
		if (rcp->rc_failover_ppa != rcp->rc_ppa)
			continue;


		printf("failover%d", dev_id);
		mode = rcp->rc_mode & (RLK_TIMEOUT_MODE|RLK_HUB_MODE);
		if (mode != 0) {
			printf("<");
			if (mode & RLK_TIMEOUT_MODE)
				printf("TIMEOUT %d", rcp->rc_rx_timeout);

			if (mode & RLK_HUB_MODE) {
				if (mode & ~RLK_HUB_MODE)
					printf(", ");

				printf("HUB");
			}
			printf(">");
		}

		printf(" = ");

		for (j=0;j<rcp->rc_num_primary_ports;j++) {
			if (j != 0)
				printf(",");

			ppa = rcp->rc_primary_ports[j];
			rc2.rc_ppa = ppa;
			rc2.rc_dev_id = -1;
			rc2.rc_type = 0;
			rc2.rc_configuration.rc_offset_ports = 0;
			if (rmapi_ctl(rmapi_fd, RMAPI_CTL_CONFIGURATION, &rc2) != 0) {
				fprintf(stderr, "RMAPI4 protocol error reading configuration for ppa %d\n", ppa);
				return;
			}
			rcp2 = &rc2.rc_configuration;

			if (rcp2->rc_num_configured_ports > 1) {
				ppa = rmapi_ppa_to_dev(ppa, RMAPI_CONFIG_TRUNK_TYPE);
				printf("trunk%d", ppa);
				if (rcp->rc_failover_port == rcp2->rc_ppa) {
					if (rcp2->rc_num_active_ports == 0)
						printf("<ACTIVE, DOWN>");
					else
						printf("<ACTIVE, UP>");
				}
				else if (rcp2->rc_num_active_ports == 0)
					printf("<DOWN>");
				else
					printf("<STANDBY>");
			}
			else {
				printf("znb%d", ppa);

				if (rcp->rc_failover_port == rcp2->rc_ppa) {
					if (rcp2->rc_linkup)
						printf("<ACTIVE, UP>");
					else
						printf("<ACTIVE, DOWN>");
				}
				else if (rcp2->rc_linkup)
					printf("<STANDBY>");
				else
					printf("<DOWN>");
			}
		}
		printf(" ==> zrl%d\n", dev_id);
	}

	if (no_rainlink)
		printf("No RAINlink features enabled\n");
}
void
rc_speedometer_set_params(ppa)
int32_t ppa;
{
	rmapi_hal_speedometer_set_params_t rh={0};
	hal_speedometer_t *hsp;
	uint32_t err;

	rh.rhss_ppa = ppa;
	hsp = &rh.rhss_speed;
	hsp->hsp_poll_rate=poll_rate; /* in milliseconds */
	hsp->hsp_version = HSP_VERSION;
	hsp->hsp_rx_pctutil=40;

	err = rmapi_ctl(rmapi_fd, RMAPI_CTL_HAL_SPEEDOMETER_SET_PARAMS, &rh);

	if (err != 0) {
		fprintf(stderr, "Error %d returned while getting state\n", err);
		return;
	}
		
}

void
rc_port_speed(ppa)
int32_t ppa;
{
	rmapi_hal_speedometer_get_t rh={0};
	hal_speedometer_t *hsp;
	uint32_t err;

	rh.rhsg_ppa = ppa;
	err = rmapi_ctl(rmapi_fd, RMAPI_CTL_HAL_SPEEDOMETER_GET, &rh);

	if (err != 0) {
		fprintf(stderr, "Error %d returned while getting state\n", err);
		return;
	}

	hsp = &rh.rhsg_speed;
		
	if (hsp->hsp_version != HSP_VERSION) {
		fprintf(stderr, "Version(%d) mismatch with HAL layer version(%d) in driver.  Need updated rlconfig program\n", HS_VERSION, hsp->hsp_version);
		return;
	}

	printf("\t\t\t\t  PORT %2d\n", ppa);
	printf("Structure Version:			%2d\n", hsp->hsp_version);
	printf("Polling rate:				%2d\n", hsp->hsp_poll_rate);
	printf("Transmit packets per sec :		%2d\n", hsp->hsp_tx_pps);
	printf("Transmit packets per sec(max):		%2d\n", hsp->hsp_tx_pps_hwm);
	printf("Transmit interrupts per sec :		%2d\n", hsp->hsp_tx_ips);
	printf("Transmit interrupts per sec(max):	%2d\n", hsp->hsp_tx_ips_hwm);
	printf("Transmit percent utilization:		%2d\n", hsp->hsp_tx_pctutil);
	printf("Transmit percent utilization(max):	%2d\n", hsp->hsp_tx_pctutil_hwm);
	printf("Transmit errors per sec :		%2d\n", hsp->hsp_tx_eps);
	printf("Transmit errors per sec(max):		%2d\n", hsp->hsp_tx_eps_hwm);
	printf("Receive  packets per sec :		%2d\n", hsp->hsp_rx_pps);
	printf("Receive  packets per sec(max):		%2d\n", hsp->hsp_rx_pps_hwm);
	printf("Receive  interrupts per sec :		%2d\n", hsp->hsp_rx_ips);
	printf("Receive  interrupts per sec(max):	%2d\n", hsp->hsp_rx_ips_hwm);
	printf("Receive  percent utilization:		%2d\n", hsp->hsp_rx_pctutil);
	printf("Receive  percent utilization(max):	%2d\n", hsp->hsp_rx_pctutil_hwm);
	printf("Receive  errors per sec :		%2d\n", hsp->hsp_rx_eps);
	printf("Receive  errors per sec(max):		%2d\n", hsp->hsp_rx_eps_hwm);


}
char *hs_state_names[] = {"HS_STATE_NOT_INIT", "HS_STATE_STOPPED", "HS_STATE_RUNNING", "HS_STATE_RELEASED"};

char *hs_fault_names[] = {"HS_FAULT_NONE", "HS_FAULT_INTERNAL", "HS_FAULT_EXTERNAL"};

char *hs_link_state_names[] = {"HS_LINK_AUTO", "HS_LINK_TP", "HS_LINK_BNC", "HS_LINK_AUI", "HS_LINK_TPFD", "HS_LINK_TX", "HS_LINK_TXFD", "HS_LINK_T4"};

void
rc_port_state(ppa)
int32_t ppa;
{
	rmapi_hal_get_state_t rh={0};
	hal_state_t *hsp;
	uint32_t err;

	rh.rhgs_ppa = ppa;
	err = rmapi_ctl(rmapi_fd, RMAPI_CTL_HAL_GET_STATE, &rh);

	if (err != 0) {
		fprintf(stderr, "Error %d returned while getting state\n", err);
		return;
	}

	hsp = &rh.rhgs_state;
		
	if (hsp->hs_version != HS_VERSION) {
		fprintf(stderr, "Version(%d) mismatch with HAL layer version(%d) in driver.  Need updated rlconfig program\n", HS_VERSION, hsp->hs_version);
		return;
	}

	printf("\t\t\t\t  PORT %2d\n", ppa);
	printf("Structure Version:		%2u\n", hsp->hs_version);
	printf("State:		%18s", hs_state_names[hsp->hs_state]);
	printf("\tFault:	%26s\n", hs_fault_names[hsp->hs_fault]);
	printf("Link configuration: %14s", hs_link_state_names[hsp->hs_link_cfg]);
	if (hsp->hs_link_state == 0)
		printf("\tCurrent link state:   HS_LINK_DOWN\n");
	else
		
		printf("\tCurrent link state: %14s\n", hs_link_state_names[hsp->hs_link_state]);
	printf("Transmit maximum resources:	%2u", hsp->hs_tx_resources_max);
	printf("\tReceive maximum resources:	%2u\n", hsp->hs_rx_resources_max);
	printf("Transmit resources in use:	%2u", hsp->hs_tx_resources);
	printf("\tReceive current resources:	%2u\n", hsp->hs_rx_resources);
	printf("Transmit resources low water:	%2u", hsp->hs_tx_lwm);
	printf("\tReceive resources low water:	%2u\n", hsp->hs_rx_lwm);
	printf("Transmit resources depleted: %5u", hsp->hs_tx_depleted);
	printf("\tReceive resources depleted: %6u\n", hsp->hs_rx_depleted);
	printf("Transmit packets:	%10u", hsp->hs_tx_packets);
	printf("\tReceive packets:	%10u\n", hsp->hs_rx_packets);
	printf("Transmit Descriptors:	%10u", hsp->hs_tx_descriptor_count);
	printf("\tReceive Descriptors:	%10u\n", hsp->hs_rx_descriptor_count);
	printf("Transmit Interrupts	%10u", hsp->hs_tx_icount);
	printf("\tReceive Interrupts	%10u\n", hsp->hs_rx_icount);
	printf("Transmit error packets:	%10u", hsp->hs_tx_err_packets);
	printf("\tReceive error packets:	%10u\n", hsp->hs_rx_err_packets);
	printf("Transmit single collisions:	%2u", hsp->hs_tx_one_collision);
	printf("\tReceive collisions:	%10u\n", hsp->hs_rx_collisions);
	printf("Transmit multiple collisions:	%2u", hsp->hs_tx_more_collisions);
	printf("\tReceive MII error:	%10u\n", hsp->hs_rx_mii_error);
	printf("Transmit maximum collisions:	%2u", hsp->hs_tx_max_collisions);
	printf("\tReceive CRC error:	%10u\n", hsp->hs_rx_crc);
	printf("Transmit late collisions:	%2u", hsp->hs_tx_late_collisions);
	printf("\tReceive descriptor error:	%2u\n", hsp->hs_rx_descriptor_error);
	printf("Transmit carrier loss:		%2u", hsp->hs_tx_carrier_loss);
	printf("\tReceive frame too long:		%2u\n", hsp->hs_rx_frame_too_long);
	printf("Transmit no carrier:		%2u", hsp->hs_tx_no_carrier);
	printf("\tReceive runt frame:	%10u\n", hsp->hs_rx_runt_frame);
	printf("Transmit jabber:		%2u", hsp->hs_tx_jabber);
	printf("\tReceive alignment error:	%2u\n", hsp->hs_rx_alignment_error);
	printf("Transmit underrun:		%2u", hsp->hs_tx_underrun);
	printf("\tReceive overrun:	%10u\n", hsp->hs_rx_overrun);
	printf("Transmit overrun:		%2u", hsp->hs_tx_overrun);
	printf("\tOther Interrupts:	%10u\n", hsp->hs_other_icount);
	printf("Transmit deferred:		%2u\n", hsp->hs_tx_deferred);
}

char *hot_swap_state[] = {"READY", "STOPPED", "RELEASED"};

char *
chipname(uint32_t chipid)
{
	switch (chipid) {
		case 0x00191011:
			return ("DEC/INTEL_21143");
		case 0x00091011:
			return ("DEC/INTEL_21140");
		case 0x00141011:
			return ("DEC/INTEL_21041");
		case 0x00021011:
			return ("DEC/INTEL_21040");
		default:
		    return ("Unknown Type");
	}
}

char *
zxename(uint32_t adapter_id)
{
	switch (adapter_id & 0xff00) {
		case 0x0A00:
			return("ZX344");
		case 0x0C00:
			return("ZX345");
		case 0x0E00:
			return("ZX346");
		case 0x0F00:
			return("ZX348");
		case 0x1200:
			return("ZX345Q");
		case 0x1300:
			return("ZX346Q");
		case 0x1400:
			return("ZX348Q");
		case 0x1800:
			return("ZX414");
		case 0x1900:
			return("ZX412");
		case 0x1A00:
			return("ZX444");
		case 0x1B00:
			return("ZX442");
		case 0x2000:
			return("ZX474");
		case 0x2100:
			return("ZX476");
		case 0x2200:
			return("ZX478");
		case 0x2300:
			return("ZX212");
		case 0x2400:
			return("ZX214");
		case 0x2800:
			return("ZX222");
		case 0x2900:
			return("ZX374");
		case 0x2A00:
			return("ZX231");
		case 0x2B00:
			return("ZX244");
		case 0x2C00:
			return("ZX424");
		case 0x2D00:
			return("ZX372");
		case 0x2E00:
			return("ZX422");
		default:
			return("Unknown");
	}
}

void
rc_pal_hw_info(ppa,fmt)
int32_t ppa;
int32_t   fmt;
{
	rmapi_pal_get_hw_info_t rp={0};
	pal_hw_info_t *psp;
	uint32_t err;

	rp.rpghi_ppa = ppa;
	err = rmapi_ctl(rmapi_fd, RMAPI_CTL_PAL_GET_HW_INFO, &rp);

	if (err != 0) {
		fprintf(stderr, "Error %d returned while getting state\n", err);
		return;
	}

	psp = &rp.rpghi_info;

	if (psp->hi_version != HI_VERSION) {
		fprintf(stderr, "Version(%d) mismatch with PAL layer version(%d) in driver.  Need updated rlstats program\n", HI_VERSION, psp->hi_version);
		return;
	}

	if (fmt ==  0) {
		int32_t a,b,c;

		printf("\t\t\t\t  PORT %2d\n", ppa);
		printf("Structure Version        %8d",psp->hi_version);

		a = psp->hi_sft_version/10000;
		b = psp->hi_sft_version - (a*10000);
		c = b/100;
		printf("\tDriver Software Version %4d.%02d.%02d\n",a,c,b-(100*c));

		if (psp->hi_state == HI_STATE_NOT_INITED) {
			printf("DEVICE NOT INSTALLED\n");
			return;
		}
			
		printf("Adapter type:        %12s", zxename(psp->hi_adapter_id));
		printf("\tSubsystem ID:             %08x\n",psp->hi_ssid);
		printf("Device type:     %16s",chipname(psp->hi_chipid));
		printf("\tBAR 0:                    %8x\n",psp->hi_bar[0]);
		printf("Device Revision:         %8x",psp->hi_revinfo & 0xff);
		printf("\tBAR 1:                    %8x\n",psp->hi_bar[1]);
		printf("Device Controls:             %04x",psp->hi_cfgcs & 0xffff);
		printf("\tBAR 2:                    %8x\n",psp->hi_bar[2]);
		printf("Device Status:               %04x",psp->hi_cfgcs>>16);
		printf("\tBAR 3:                    %8x\n",psp->hi_bar[3]);
		printf("Latency Timer:           %8d",(psp->hi_ltcls>>8) & 0xff);
		printf("\tBAR 4:                    %8x\n",psp->hi_bar[4]);
		printf("Cache Line Size:         %8d",psp->hi_ltcls & 0xff);
		printf("\tBAR 5:                    %8x\n",psp->hi_bar[5]);
		printf("Irq                      %8d",psp->hi_irq & 0xff);
		printf("\tROM Base Address:         %8x\n",psp->hi_rombar);
		printf("Bus:                     %8d",psp->hi_bus);
		printf("\tFactory Address  %02x:%02x:%02x:%02x:%02x:%02x\n",
			psp->hi_macaddr[0]>>24&0xff,psp->hi_macaddr[0]>>16&0xff,
			psp->hi_macaddr[0]>>8&0xff,psp->hi_macaddr[0]&0xff,
			psp->hi_macaddr[1]>>24&0xff,psp->hi_macaddr[1]>>16&0xff);
		printf("Device Number:           %8d",psp->hi_devno);
		printf("\tCurrent Address  %02x:%02x:%02x:%02x:%02x:%02x\n",
			psp->hi_macaddr[1]>>8&0xff,psp->hi_macaddr[1]&0xff,
			psp->hi_macaddr[2]>>24&0xff, psp->hi_macaddr[2]>>16&0xff,
			psp->hi_macaddr[2]>>8&0xff,psp->hi_macaddr[2]&0xff);
		printf("\n"); 
	
	} else {	/* Raw output form, all values on one line, separated by a space*/
		printf("%2d %d %d ",ppa,psp->hi_state,psp->hi_sft_version);
		printf("%2d %2d ",psp->hi_bus,psp->hi_devno);
		printf("%04x %08x ",psp->hi_adapter_id,psp->hi_chipid);
		printf("%04x %04x ",psp->hi_cfgcs>>16,psp->hi_cfgcs & 0xffff);
		printf("%06x %02x ",psp->hi_revinfo>>8,psp->hi_revinfo & 0xff);
		printf("%02x %02x ",(psp->hi_ltcls>>8)&0xff, psp->hi_ltcls&0xff);
		printf("%x %x %x ",psp->hi_bar[0],psp->hi_bar[1],psp->hi_bar[2]);
		printf("%x %x %x ",psp->hi_bar[3],psp->hi_bar[4],psp->hi_bar[5]);
		printf("%x %d %d %d %d ",psp->hi_rombar,psp->hi_irq>>24&0xff,
			psp->hi_irq>>16&0xff,psp->hi_irq>>8&0xff,psp->hi_irq&0xff);
		printf("%02x:%02x:%02x:%02x:%02x:%02x ",
			psp->hi_macaddr[0]>>24&0xff,psp->hi_macaddr[0]>>16&0xff,psp->hi_macaddr[0]>>8&0xff,
			psp->hi_macaddr[0]&0xff,psp->hi_macaddr[1]>>24&0xff,psp->hi_macaddr[1]>>16&0xff);
		printf("%02x:%02x:%02x:%02x:%02x:%02x\n",
			psp->hi_macaddr[1]>>8&0xff,psp->hi_macaddr[1]&0xff,psp->hi_macaddr[2]>>24&0xff,
			psp->hi_macaddr[2]>>16&0xff,psp->hi_macaddr[2]>>8&0xff,psp->hi_macaddr[2]&0xff);
	}
}

usage(s)
char *s;
{
	printf("%s\n", s);
	printf("Usage: rlstats [[-m|-s|-o|-i] <ppa>] [-d <level>] [-z] \n");
	printf("                -m: MAC layer statistics on specified <ppa>\n");
	printf("                -s: Turn speedometer on specified <ppa>\n");
	printf("                -r: Speedometer polling rate (in ms) on -s <ppa>\n");
	printf("                -i: Hardware information for specified <ppa>\n");
	printf("                -o: Display speedometer statistics on specified <ppa> \n");
	printf("                -d: set debug level to <level>\n");
	printf("                -z: Show RAINlink trunk configuration\n");
}

