%{
#include <stdio.h>
#include <pdk_port.h>
#include <pdk_global.h>
#include <pdk_types.h>
#include <pdk_errno.h>
#include <pdk_psdb.h>

#include <rlconfig.h>
extern int lineno;

%}

%union {
	int32_t ppa;
	ppa_array_t *ppa_array;
}

%type <ppa_array> znblist
%type <ppa> trunk failover znb mode hex_number dec_number number

%token <ppa> DIGIT
%token HEX
%token SEMICOLON
%token EQUAL
%token TRUNK
%token FAILOVER
%token ZNB
%token COMMA
%token NOT_EQUAL
%token HUB_MODE
%token SNAP_BCAST_MODE
%token ROUND_ROBIN_MODE
%token IP_TRUNKING_MODE
%token BALANCE_MODE
%token INTEGRITY_MODE
%token LAYER_3_MODE
%token LAYER_2_MODE
%token TIMEOUT_MODE
%token TEARDOWN

%%

statements:	statement statements
	|	statement
	;

statement:	trunk EQUAL znblist SEMICOLON
		{
			if ($3->pa_has_trunk != 0) {
				fprintf(stderr, "line %d: ", lineno);
				fprintf(stderr, "trunk%d - Trunks cannot have trunks\n", $1);
				exit(0);
			}
			if ($3->pa_index < 2) {
				fprintf(stderr, "line %d: ", lineno);
				fprintf(stderr, "trunk%d - Must have two or more elements\n", $1);
				exit(0);
			}
			rc_create_trunk($1, $3);
		}
	|	failover EQUAL znblist SEMICOLON
		{
			if ($3->pa_index < 2) {
				fprintf(stderr, "line %d: ", lineno);
				fprintf(stderr, "failover%d - Must have two or more elements\n", $1);
				exit(0);
			}
			rc_create_failover($1, $3);
		}
	|	trunk EQUAL mode SEMICOLON
		{
			rc_set_mode($1, $3);
		}
	|	trunk NOT_EQUAL mode SEMICOLON
		{
			rc_clear_mode($1, $3);
		}
	|	trunk TEARDOWN SEMICOLON
		{
			rc_teardown_trunk($1);
		}
	|	failover EQUAL mode SEMICOLON
		{
			rc_set_mode($1, $3);
		}
	|	failover NOT_EQUAL mode SEMICOLON
		{
			rc_clear_mode($1, $3);
		}
	|	failover EQUAL TIMEOUT_MODE number SEMICOLON
		{
			rc_set_timer_mode($1, $4);
		}
	|	failover NOT_EQUAL TIMEOUT_MODE SEMICOLON
		{
			rc_clear_mode($1, RLK_TIMEOUT_MODE);
		}
	|	failover TEARDOWN SEMICOLON
		{
			rc_teardown_failover($1);
		}
	;

znblist:	znb COMMA znblist
		{
			if (rc_push_ppa($3, $1) != 0) {
				yyerror("Too many ports to group\n");
			}

			$$ = $3;
		}
	|	trunk COMMA znblist
		{
			ppa_array_t *pa;

			if ((pa = rc_find_group(0, $1, 0)) == 0) {
				fprintf(stderr, "Reference to undefined trunk %d - pa 0\n", $1);
				exit(0);
			}
			if (pa->pa_is_trunk == 0) {
				fprintf(stderr, "Reference to undefined trunk %d - pa_is_trunk 0\n", $1);
				exit(0);
			}
			/* RAINlink layer references trunks by lowest valued
			 * ppa.  The first entry in the array is lowest value.
			 * because they are stored in an ordered list
			 */

			if (rc_trunk_in_use($1) != 0) {
				fprintf(stderr, "line %d: ", lineno);
				fprintf(stderr, "trunk%d in use\n", $1);
				exit(0);
			}

			if (rc_push_ppa($3, pa->pa_array[0]) != 0) {
				yyerror("Too many ports to group\n");
			}

			$3->pa_has_trunk = 1;
			$$ = $3;
		}
	|	znb
		{
			ppa_array_t *pa;

			pa = rc_create_ppa_array();
			if (pa == 0) {
				yyerror("Could not allocate ppa_array\n");
			}

			rc_push_ppa(pa, $1);
			$$ = pa;
		}
	|	trunk
		{
			ppa_array_t *pa, *trunk_pa;

			if ((trunk_pa = rc_find_group(0, $1, 0)) == 0) {
				yyerror("Reference to undefined trunk\n");
			}
			if (trunk_pa->pa_is_trunk == 0) {
				yyerror("Reference to undefined trunk\n");
			}
			/* RAINlink layer references trunks by lowest valued
			 * ppa.  The first entry in the array is lowest value.
			 * because they are stored in an ordered list
			 */
			if (rc_trunk_in_use($1) != 0) {
				fprintf(stderr, "line %d: ", lineno);
				fprintf(stderr, "trunk%d in use\n", $1);
				exit(0);
			}

			pa = rc_create_ppa_array();
			if (pa == 0) {
				yyerror("Could not allocate ppa_array\n");
			}

			rc_push_ppa(pa, trunk_pa->pa_array[0]);
			pa->pa_has_trunk = 1;
			$$ = pa;
		}
	;

mode:		ROUND_ROBIN_MODE
		{
			$$ = RLK_ROUND_ROBIN_MODE;
		}
	|	IP_TRUNKING_MODE
		{
			$$ = RLK_IP_TRUNKING_MODE;
		}
	|	BALANCE_MODE
		{
			$$ = RLK_BALANCE_MODE;
		}
	|	INTEGRITY_MODE
		{
			$$ = RLK_INTEGRITY_MODE;
		}
	|	LAYER_3_MODE
		{
			$$ = RLK_LAYER_3_MODE;
		}
	|	LAYER_2_MODE
		{
			$$ = RLK_LAYER_2_MODE;
		}
	|	HUB_MODE
		{
			$$ = RLK_HUB_MODE;
		}
	|	SNAP_BCAST_MODE
		{
			$$ = RLK_SNAP_MODE;
		}
	;

trunk:		TRUNK number
		{
			$$ = $2;
		}
	;

znb:		ZNB number
		{
			if (rc_ppa_in_use($2) != 0) {
				fprintf(stderr, "line %d: ", lineno);
				fprintf(stderr, "znb%d in use\n", $2);
				exit(0);
			}

			$$=$2;
		}
	;

failover:	FAILOVER number
		{
			$$=$2;
		}
	;

/*  Common rules */

number:		dec_number
		{
			$$ = $1;
		}
	|	HEX hex_number
		{
			$$ = $2;
		}
	;
		

dec_number:	dec_number DIGIT
		{
			$$ = (10*$1) + $2;
		}
	|	DIGIT
		{
			$$=$1;
		}

hex_number:	hex_number DIGIT
		{
			$$ = (16*$1) + $2;
		}
	|	DIGIT
		{
			$$=$1;
		}
%%

