/*==========================================================================
**
**	./pdk/src/drivers/sal/sal_queue.c
**
**	System abstraction layer
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	ram	04/01/98 	Created
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/

#include <pdk_port.h>
#undef _KERNEL
#include <pdk_global.h>
#include <pdk_types.h>
#include <pdk_queue.h>

/* queueing functions */
void
pdk_q_enq(q, e)
pdk_q_t **q;
pdk_q_t *e;
{
	if (*q == 0)
		*q = e;

	e->pdk_q_link = (*q)->pdk_q_link;
	(*q)->pdk_q_link = e;
	*q = e;
}

void
pdk_q_fnq(q, e)
pdk_q_t **q;
pdk_q_t *e;
{
	if (*q == 0)
		*q = e;

	e->pdk_q_link = (*q)->pdk_q_link;
	(*q)->pdk_q_link = e;
}

pdk_q_t *
pdk_q_deq(q)
pdk_q_t **q;
{
	pdk_q_t *e;

	if (*q == 0)
		return(0);

	e = (*q)->pdk_q_link;
	if (e == *q)
		*q = 0;
	else
		(*q)->pdk_q_link = e->pdk_q_link;

	return(e);
}

pdk_q_t *
pdk_q_first(q)
pdk_q_t **q;
{
	if (*q == 0)
		return(0);

	return((*q)->pdk_q_link);
}

pdk_q_t *
pdk_q_next(q, e)
pdk_q_t **q;
pdk_q_t *e;
{
	if (*q == e)
		return(0);

	if (e == 0)
		return((*q)->pdk_q_link);

	return(e->pdk_q_link);
}

void
pdk_q_insert(q, p, e)
pdk_q_t **q;
pdk_q_t *p;
pdk_q_t *e;
{
	if (*q == 0) {
		*q = e;
		e->pdk_q_link = e;
		return;
	}

	/* If p is zero insert at beginning of queue */
	if (p == 0)
		p = *q;
	else if (*q == p)   /* else if last element update q head */
		*q = e;

	e->pdk_q_link = p->pdk_q_link;
	p->pdk_q_link = e;
}

pdk_q_t *
pdk_q_extract(q, e)
pdk_q_t **q;
pdk_q_t *e;
{
	pdk_q_t *p;
	
	if (*q == 0)
		return(0);

	p = 0;
	while ((p = pdk_q_next(q, p)) != 0) {
		if (p->pdk_q_link == e) {
			pdk_q_extractp(q, p);
			return(p);
		}
	}

	return(0);
}

/*  Extract element given the previous */
void
pdk_q_extractp(q, p)
pdk_q_t **q;
pdk_q_t *p;
{
	if (*q == 0)
		return;

	/* If element is zero extract the first */
	if (p == 0)
		p = *q;

	if (p->pdk_q_link == *q) {
		if (p == *q) {
			*q = 0;
			return;
		}

		*q = p;
	}

	p->pdk_q_link = p->pdk_q_link->pdk_q_link;
}

