/*==========================================================================
**
**	znyx/deviceid.h
**
**	ZNYX Device Identifiers
**	$Revision: 1.2 $
**
**==========================================================================
**
**	(c) Copyright 1995-1999 ZNYX Corporation
**	(c) Copyright 1999-2002 ZNYX Networks
**	All Rights Reserved.
**
**==========================================================================
**
**	Modification History:
**	jr	10/04/95	Created from SD0008 document
**	jr	10/25/95	Added new models: 351, 345, 311
**	jr	01/03/96	Added new models: 346, 348
**	jr	05/28/96	Changed comment style for gcc
**	dew	12/03/96	Added model: 345-X2 for ICS PHY
**	jr	01/29/97	Added 21142, 21143, 6692
**	jr	02/03/97	Added ZX348-01
**	dew	08/27/97	Added ZX345Q and ZX346Q
**	jr	08/29/97	Added ZX348Q
**	dew	01/29/98	Added Quality 6612
**	dew	02/27/98	Added zx150 PCI devices
**	ram	03/03/98	Added ZX414
**	ram	03/13/98	Added ZX412
**	ram	03/13/98	Added National 83843
**	ram	03/18/98	Added SUBDEVICEID macro
**	dew	08/18/98	Added zx160
**	dew	09/02/98	Added zx444 and 442
**	dew	09/22/98	Added zx361
**	dew	11/23/98	Added zx361_x2
**	dew	02/08/99	Added zx474, zx476, 478
**	drc	03/22/99	Added zx400, zx400ar, zx400hr
**	drc	04/19/99	Added zx212 zx214
**	drc	06/07/99	Added zx458 zx459
**	drc	10/12/99	Added zx222
**	drc	10/21/99	Added zx374
**	bg	12/22/99	Added zx231 zx244
**	bg	02/22/00	Added zx424
**	bg	03/01/00	Added zx372 422
**	bg	03/09/00	Added 374 BOOTROM distinction, 4500 numbers
**	bg	09/20/00	Added 231 9500 byte
**	bg	10/23/00	Added BCM5218
**	bg	06/28/01	Added ZX4600 numbers
**	bg	09/07/01	Fixed 4500P comment; added ZX4526P, BCM5228
**	jr	09/16/01	Added 82555, 82557, 82559, 82543, 82544
**	bg	09/24/01	Added BCM5402
**	dew	11/29/01	Intel bridge conversion numbers
**	dew	01/17/02	Added Zx4524P3 for the 4500P with 5615
**	dew	01/21/02	Added Zx4526P3 for the 4500P with 5615 (gig)
**==========================================================================
*/

#ifndef DEVICEID_H
#define DEVICEID_H

#define SUBVENDORID_ZNYX	0x110D


#define SUBDEVICEID(adapter_id)	(((adapter_id) >> 8) &0xff)

#define ADAPTER_ZX_MASK		0xFF00	/* ZX-level mask for models*/

#define ADAPTER_ZX100		0x0001	/* ZX100 */
#define ADAPTER_ZX110		0x0002	/* ZX110 */
#define ADAPTER_ZX300		0x0300	/* All models */
#define ADAPTER_ZX301		0x0400	/* Revision C,D only */
#define ADAPTER_ZX302		0x0500	/* All models */
#define ADAPTER_ZX312		0x0600	/* All models */
#define ADAPTER_ZX312L6		0x0601	/* Legacy6 PROM */
#define ADAPTER_ZX312L32	0x0602	/* Legacy32 PROM */
#define ADAPTER_ZX312T		0x0622	/* 10Base-T only version */
#define ADAPTER_ZX314		0x0700	/* All models */
#define ADAPTER_ZX314SI		0x0701	/* INTA only configuration */
#define ADAPTER_ZX314RI		0x0711	/* INTA/B/C/D configuration */
#define ADAPTER_ZX315		0x0800	/* All models */
#define ADAPTER_ZX315SI		0x0801	/* INTA only configuration */
#define ADAPTER_ZX315RI		0x0811	/* INTA/B configuration */

#define ADAPTER_ZX342		0x0900	/* All models */
#define ADAPTER_ZX342X2		0x0901	/* X1, X2 */
#define ADAPTER_ZX342B		0x0921	/* 100 Mbps only version */
#define ADAPTER_ZX344		0x0A00	/* All models */
#define ADAPTER_ZX344X2		0x0A01	/* X1, X2 */
#define ADAPTER_ZX351		0x0B00	/* All models */
#define ADAPTER_ZX351X1		0x0B01	/* X1 */
#define ADAPTER_ZX345		0x0C00	/* All models */
#define ADAPTER_ZX345X1		0x0C01	/* X1 */
#define ADAPTER_ZX345X2		0x0C02	/* X2 uses ICS 1890 PHY */
#define ADAPTER_ZX345F		0x0C03	/* uses National 83843A PHY */
#define ADAPTER_ZX345FA		0x0C04	/* with AUI connector */
#define ADAPTER_ZX311		0x0D00	/* All models */
#define ADAPTER_ZX311X1		0x0D01	/* X1 */
#define ADAPTER_ZX346		0x0E00	/* All models */
#define ADAPTER_ZX346X1		0x0E01	/* X1 */
#define ADAPTER_ZX348		0x0F00	/* All models */
#define ADAPTER_ZX348X1		0x0F01	/* X1 (National DP83840) */
#define ADAPTER_ZX348_01	0x0F02	/* 01 (ICS 1890) */
#define ADAPTER_ZX130		0x1000
#define ADAPTER_ZX318		0x1100
#define ADAPTER_ZX345AP		0x1200	/* All models (obsolete) */
#define ADAPTER_ZX345AP_X1	0x1201	/* X1 (obsolete) */
#define ADAPTER_ZX345AP_X2	0x1202	/* X2 (obsolete) */
#define ADAPTER_ZX345Q		0x1200	/* All models */
#define ADAPTER_ZX345Q_X2	0x1202	/* DEC 21143 & Quality 6611 Phy */
#define ADAPTER_ZX346Q		0x1300	/* DEC 21143 & Quality 6611 Phy */
#define ADAPTER_ZX346Qi		0x1301	/* Zx346Q, Intel 21152 bridge */
#define ADAPTER_ZX348Q		0x1400	/* DEC 21143 & Quality 6611 Phy */
#define ADAPTER_ZX348Qi		0x1401	/* Zx348Q, Intel 21152 bridge */
#define ADAPTER_ZX150		0x1500	/* DEC 21143 & Quality 6612 Phy */
#define DEVID_ZX150_DRAM	0x1600	/* DRAM controller */
#define DEVID_ZX150_CPI		0x1700	/* Coprocessor PCI Interface */
#define ADAPTER_ZX414		0x1800	/* All models */
#define ADAPTER_ZX412		0x1900	/* All models */
#define ADAPTER_ZX444		0x1A00	/* All models */
#define ADAPTER_ZX444i		0x1A01	/* Zx444, Intel 21555 bridge */
#define ADAPTER_ZX442		0x1B00	/* All models */
#define ADAPTER_ZX442i		0x1B01	/* Zx442, Intel 21555 bridge */
#define ADAPTER_ZX160_TP	0x1C00	/* All models 21041 ports */
#define ADAPTER_ZX160_TX	0x1D00	/* All models 21143 ports */
#define ADAPTER_ZX160_ATM	0x1E00	/* All models ATM port */
#define ADAPTER_ZX361		0x1F00	/* DEC 21143 & Quality 6612 Phy */
#define ADAPTER_ZX361_X2	0x1F02	/* DEC 21143, Broadcom 5201 Phy */
#define ADAPTER_ZX474		0x2000	/* DEC 21143, Broadcom 5208 Phy */
#define ADAPTER_ZX474i		0x2001	/* Zx474, Intel 21555 bridge */
#define ADAPTER_ZX476		0x2100	/* DEC 21143, Broadcom 5208 Phy */
#define ADAPTER_ZX478		0x2200	/* DEC 21143, Broadcom 5208 Phy */
#define ADAPTER_ZX478i		0x2201	/* Zx478, Intel 21555 bridge */
#define ADAPTER_ZX212		0x2300	/* PMC DEC 21143, Quality 6611 */
#define ADAPTER_ZX212i		0x2301	/* Zx212, Intel 21152 bridge */
#define ADAPTER_ZX214		0x2400	/* PMC DEC 21143, Quality 6611 */
#define ADAPTER_ZX214i		0x2401	/* Zx214, Intel 21152 bridge */

#define ADAPTER_ZX400		0x2500	/* Compact PCI RTM, All models */
#define ADAPTER_ZX400AR		0x2501	/* CPCI RTM, 404AR, 408AR */
#define ADAPTER_ZX400HR		0x2502	/* CPCI RTM, 404HR, 408HR */
#define ADAPTER_ZX4524		0x2600	/* cPCI Broadcom 5600, BCM 5208 */
#define ADAPTER_ZX4524i		0x2601	/* Zx4500, Intel 21555 bridge */
#define ADAPTER_ZX4524g		0x2700	/* cPCI Broadcom 5600, gigabit */

#define ADAPTER_ZX222		0x2800	/* PMC DEC 21143, Broadcom 5201 Phy */
#define ADAPTER_ZX374		0x2900	/* DEC 21143 & Kendin ROM */
#define ADAPTER_ZX374_A		0x2901	/* DEC 21143 & Kendin no ROM */
#define ADAPTER_ZX374i		0x2902	/* Zx374, Intel 21154 bridge */

#define ADAPTER_ZX231		0x2A00	/* Single port, PMC Gigabit */
#define ADAPTER_ZX244		0x2B00	/* 4 Port PMC PIM */
#define ADAPTER_ZX244i		0x2B01	/* Zx244, Intel 21154 bridge */

#define ADAPTER_ZX424		0x2C00	/* cPCI, 21154, 21143, Kendin */
#define ADAPTER_ZX424i		0x2C01	/* Zx424, Intel 21154 bridge */

#define ADAPTER_ZX372		0x2D00	/* PCI, 21154, 21143, Kendin, ROM */
#define ADAPTER_ZX372_A		0x2D01	/* PCI, 21154, 21143, Kendin, no ROM */
#define ADAPTER_ZX372i		0x2D02	/* Zx372, Intel 21154 bridge */
#define ADAPTER_ZX422		0x2E00	/* cPCI, 21154, 21143, Kendin */
#define ADAPTER_ZX422i		0x2E01	/* Zx422, Intel 21154 bridge */

#define ADAPTER_ZX231J		0x2F00	/* Single port, PMC Gigabit 9500 MTU*/

#define ADAPTER_ZX4608		0x3000	/* cPCI BCM5680, gigabit */
#define ADAPTER_ZX4500P		0x3100	/* cPCI BCM5605, PICMG 2.16 */
#define ADAPTER_ZX4500Pi	0x3101	/* Zx4500P, Intel 21555 bridge */
#define ADAPTER_ZX4524P3	0x3103	/* BCM 5615, MOT 8240, Intel 21555 */
#define ADAPTER_ZX4526P		0x3200	/* cPCI BCM5605, gigabit */
#define ADAPTER_ZX4526P3	0x3203	/* BCM 5615, MOT 8240, 21555, gigabit */

#define DEVICE_DEC21040		21040	/* DEC 21040 */
#define DEVICE_DEC21041		21041	/* DEC 21041 */
#define DEVICE_DEC21140		21140	/* DEC 21140 */
#define DEVICE_DEC21142		21142	/* DEC 21142 */
#define DEVICE_DEC21143		21143	/* DEC 21143 */
#define DEVICE_PSX130		130	/* pciSwitch Accelerator ZX130 */
#define DEVICE_DP83840		840	/* National DP 83840 */
#define DEVICE_DP83843		843	/* National DP 83843 */
#define DEVICE_AMD865		865	/* AMD 865/866 */
#define DEVICE_LXT970		970	/* Level One LXT970 */
#define DEVICE_ICS1890		1890	/* ICS 1890 */
#define DEVICE_BCM5000		5000	/* Broadcom BCM5000 */
#define DEVICE_BCM5201		5201	/* Broadcom BCM5201 */
#define DEVICE_BCM5208		5208	/* Broadcom BCM5208 */
#define DEVICE_BCM5218		5218	/* Broadcom BCM5218 */
#define DEVICE_BCM5228		5228	/* Broadcom BCM5228 */
#define DEVICE_BCM5402		5402	/* Broadcom BCM5402 */
#define DEVICE_BCM5600		5600	/* Broadcom BCM5600 */
#define DEVICE_BCM5605		5605	/* Broadcom BCM5605 */
#define DEVICE_BCM5615		5615	/* Broadcom BCM5615 */
#define DEVICE_BCM5680		5680	/* Broadcom BCM5680 */
#define DEVICE_QS6611		6611	/* Quality Semiconductor 6611 */
#define DEVICE_QS6612		6612	/* Quality Semiconductor 6612 */
#define DEVICE_ML6692		6692	/* Micro Linear 6692 */
#define DEVICE_XQ11800		11800	/* Vitesse Gigabit Mac */
#define DEVICE_82555		555	/* Intel 82555 */
#define DEVICE_82557		557	/* Intel 82557 */
#define DEVICE_82558		558	/* Intel 82558 */
#define DEVICE_82559		559	/* Intel 82559 */
#define DEVICE_82543		543	/* Intel 82543 */
#define DEVICE_82544		544	/* Intel 82544 */

#endif DEVICEID_H

