/*==========================================================================
**
**	znyx/hal/pdk_hal.h.h
**
**	Hardware abstraction layer interface definition
**
**==========================================================================
**
**	(c) Copyright 1998, 1999 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**	ram	04/28/98	Created
**	jr	02/06/02	added rx/tx byte count
**
**==========================================================================
*/
#include <pdk_buf.h>
#include <pdk_layer.h>

/* Hardware abstraction layer interface */

/* pdk_xmit_complete status */
#define PDK_XMIT_STATUS_ERR 1
#define PDK_XMIT_STATUS_FLUSHING 2

typedef struct pdk_hal_stat_s {
	uint32_t	hs_tx_bytes;
	uint32_t	hs_tx_packets;
	uint32_t	hs_tx_err_packets;
	uint32_t	hs_tx_one_collision;
	uint32_t	hs_tx_more_collisions;
	uint32_t	hs_tx_max_collisions;
	uint32_t	hs_tx_late_collisions;
	uint32_t	hs_tx_carrier_loss;
	uint32_t	hs_tx_no_carrier;
	uint32_t	hs_tx_jabber;
	uint32_t	hs_tx_underrun;
	uint32_t	hs_tx_overrun;
	uint32_t	hs_tx_deferred;
	uint32_t	hs_rx_bytes;
	uint32_t	hs_rx_packets;
	uint32_t	hs_rx_err_packets;
	uint32_t	hs_rx_collisions;
	uint32_t	hs_rx_mii_error;
	uint32_t	hs_rx_crc;
	uint32_t	hs_rx_descriptor_error;
	uint32_t	hs_rx_frame_too_long;
	uint32_t	hs_rx_runt_frame;
	uint32_t	hs_rx_alignment_error;
	uint32_t	hs_rx_overrun;
} pdk_hal_stat_t;

/* ioctls */
#define	PDK_HAL_GET_INFLIGHT			0
#define PDK_HAL_GETID				1
#define PDK_HAL_SETID				2
#define PDK_HAL_INIT_AND_START			3
#define PDK_HAL_STOP_AND_RESET			4
#define PDK_HAL_DEBUG				5
#define PDK_HAL_REVLEVEL			6
#define PDK_HAL_BURST_LIMIT			7
#define PDK_HAL_GET_COUNTERS			8
#define PDK_HAL_SET_COUNTERS			9
#define PDK_HAL_GET_PHYSADDR			10
#define PDK_HAL_SET_PHYSADDR			11
#define PDK_HAL_CLEAR_PROMISCUOUS		12
#define PDK_HAL_SET_PROMISCUOUS_MULTI		13
#define PDK_HAL_SET_PROMISCUOUS_PHYS		14
#define PDK_HAL_ADD_MULTICAST			15
#define PDK_HAL_DEL_MULTICAST			16
#define PDK_HAL_CLEAR_MULTICAST			17
#define PDK_HAL_DUMPCAM				18
#define PDK_HAL_DUMPRING			19
#define PDK_HAL_LED				20
#define PDK_HAL_SET_MEDIATYPE			21
#define PDK_HAL_ENABLE_INTERRUPTS		22
#define PDK_HAL_DISABLE_INTERRUPTS		23
#define PDK_HAL_GET_NUM_RECEIVE_BUFS		24
#define PDK_HAL_RECEIVE_INIT			25
#define PDK_HAL_RECEIVE_DEINIT			26
#define PDK_HAL_MTU_SIZE			27
#define PDK_HAL_GET_MEDIATYPE			28
#define PDK_HAL_EXT_FAULT			29
#define PDK_HAL_RESTART				30
#define PDK_HAL_SET_TX_PENDING_THRESHOLD	31
#define PDK_HAL_HOTSWAP				32
#define PDK_HAL_SET_SPEEDOMETER_POLL_RATE_AND_START	33
#define PDK_HAL_STOP_SPEEDOMETER_POLL		34
#define PDK_HAL_GET_HDWRADDR			35

/* status opcodes */
#define PDK_STATUS_LINK_DOWN			0
#define PDK_STATUS_LINK_UP			1
#define PDK_STATUS_LINK_CHANGE			2
#define PDK_STATUS_INTERNAL_FAULT		3
#define PDK_STATUS_EXTERNAL_FAULT		4


/* Media Types */
#define ZXE_MEDIA_AUTO		0
#define ZXE_MEDIA_TP		1
#define ZXE_MEDIA_BNC		2
#define ZXE_MEDIA_AUI		3
#define ZXE_MEDIA_TPFD		4
#define ZXE_MEDIA_TX		5
#define ZXE_MEDIA_TXFD		6
#define ZXE_MEDIA_T4		7
#define ZXE_MEDIA_AUTO_COMPLETE	8
#define ZXE_MEDIA_MAX		8

uint32_t pdk_hal_global_init();
uint32_t pdk_hal_global_deinit();
uint32_t pdk_hal_init(int32_t ppa);
uint32_t pdk_hal_deinit(int32_t ppa);

pdk_layer_routines_t *pdk_hal_register(int32_t ppa, pdk_layer_routines_t *up, pdk_layer_routines_t *down);

uint32_t pdk_hal_ctl(int32_t ppa, uint32_t opcode, void *param);

