/*
**
**	./pdk/src/drivers/pal/pio_port.h
**
**	PIO port specific header file - Linux version
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**==========================================================================
**
**	Modification History:
**	Tim Hayes tim.hayes@znyx.com Spring 2K
**
**==========================================================================
**
**	Notes:
**==========================================================================
*/
#ifndef PDK_PIO_PORT_H
#define PDK_PIO_PORT_H

#ifdef __KERNEL__
#include <linux/skbuff.h>
#include <linux/sockios.h>
#endif

#define ZNBIOC_ENAB_INTS	SIOCDEVPRIVATE

#define SIOZSTART	SIOCDEVPRIVATE
/* sync I/O */
#define SIOZOPEN	SIOZSTART	
#define SIOZREAD	SIOZSTART+1	
#define SIOZWRITE	SIOZSTART+2	
#define SIOZCLOSE	SIOZSTART+3	

#define SIOZBLOPEN	SIOZSTART+4	

#define pio_sleep_init(pio) { \
		(pio)->pio_open_proc = current; \
		(pio)->pio_pid = current->pid; \
		init_waitqueue_head(&((pio)->pio_sleep_arg)); \
}
#define pio_sleep_deinit(pio)
#define pio_wakeup(pio) wake_up_interruptible(&((pio)->pio_sleep_arg))

extern void rtnl_unlock();
extern void rtnl_lock();

#define pio_sleep(pio,B,C,D) { \
	rtnl_unlock(); \
	interruptible_sleep_on(&((pio)->pio_sleep_arg)); \
	rtnl_lock(); \
}

#define pio_signal_setup(pio, sig) { \
	(pio)->pio_signal = (sig); \
	(pio)->pio_proc = current; \
}

#define pio_signal(pio) (send_sig((pio)->pio_signal, (pio)->pio_proc, 1))
	
		
/** #if (LINUX_VERSION_CODE < 0x020212) change appeared between 2.2.17 and 2.2.18 **/

#ifndef DECLARE_WAIT_QUEUE_HEAD
typedef struct wait_queue *wait_queue_head_t;
#define init_waitqueue_head(x)	do { *(x) = NULL; } while (0)
#endif
#define pio_sleep_arg_t wait_queue_head_t
#define pio_proc_t struct task_struct
#define pio_msg_alloc(size) dev_alloc_skb(size)
#define pio_msg_free(A) dev_free_skb(A)
#define pio_msg_freem(A) dev_free_skb(A)
#define pio_msg_buf_t struct sk_buff
#define pio_msg_len len
#define pio_msg_next next
#define pio_msg_data data
#define pio_msg_copy(A,B,C) pdk_bcopy(A,B,C)
#define pio_msg_dup(m) skb_copy(m, 0)

#endif /* PDK_PIO_PORT_H */

