/*==========================================================================
**
**	./pdk/src/drivers/sal/sal_reg.c
**
**	System abstraction layer for Linux
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**==========================================================================
**
**	david	02/16/99 	Created
**	jr	09/19/01	NB version, namespace cleanup
**	jr	01/28/02	removed pdk_reg_get32() big endian archs
**
**==========================================================================
*/

#include <pdk_port.h>

#include <pdk_global.h>
#include <pdk_types.h>
#include <pdk_linux.h>
#include <pdk_reg.h>
#include <pdk_print.h>
#include <pdk_hton.h>

#include <asm/io.h>

/*extern int32_t sal_debug;*/

#define DEBUG1 if (pdk_sal_debug>0) pdk_print
#define DEBUG2 if (pdk_sal_debug>1) pdk_print
#define DEBUG3 if (pdk_sal_debug>2) pdk_print

#ifdef BR554
#define pdk_reg_config_put8 pdk_raw_config_put8
#define pdk_reg_config_put16 pdk_raw_config_put16
#define pdk_reg_config_put32 pdk_raw_config_put32
#define pdk_reg_config_get8 pdk_raw_config_get8
#define pdk_reg_config_get16 pdk_raw_config_get16
#define pdk_reg_config_get32 pdk_raw_config_get32
#endif


void
pdk_reg_global_init()
{
	int32_t i;

	for(i = 0; i < NUM_PORTS; i++) {
		znb_pdev[i] = 0;
	}
}

void
pdk_reg_config_put8(ppa, offset, value)
int32_t ppa;
uint32_t offset;
uchar value;
{
	
	if (znb_pdev[ppa] == 0) {
		DEBUG1("pdk_reg_config_put8:  Invalid ppa(%d)", ppa);
		return;
	}

	pci_write_config_byte(znb_pdev[ppa], offset, value);
}

void
pdk_reg_config_put16(ppa, offset, value)
int32_t ppa;
uint32_t offset;
ushort value;
{
	
	if (znb_pdev[ppa] == 0) {
		DEBUG1("pdk_reg_config_put16:  Invalid ppa(%d)", ppa);
		return;
	}

	pci_write_config_word(znb_pdev[ppa], offset, value);
}

void
pdk_reg_config_put32(ppa, offset, value)
int32_t ppa;
uint32_t offset;
uint32_t value;
{
	if (znb_pdev[ppa] == 0) {
		DEBUG1("pdk_reg_config_put32:  Invalid ppa(%d)", ppa);
		return;
	}

	pci_write_config_dword(znb_pdev[ppa], offset, value);
}

uchar
pdk_reg_config_get8(ppa, offset)
int32_t ppa;
uint32_t offset;
{
	uchar value;
	
	if (znb_pdev[ppa] == 0) {
		DEBUG1("pdk_reg_config_get8:  Invalid ppa(%d)", ppa);
		return;
	}

	pci_read_config_byte(znb_pdev[ppa], offset, &value);
	return(value);
}

ushort
pdk_reg_config_get16(ppa, offset)
int32_t ppa;
uint32_t offset;
{
	ushort value;
	
	if (znb_pdev[ppa] == 0) {
		DEBUG1("pdk_reg_config_get16:  Invalid ppa(%d)", ppa);
		return;
	}

	pci_read_config_word(znb_pdev[ppa], offset, &value);
	return(value);
}


uint32_t
pdk_reg_config_get32(ppa, offset)
int32_t ppa;
uint32_t offset;
{
	uint32_t value;
	
	if (znb_pdev[ppa] == 0) {
		DEBUG1("pdk_reg_config_get32:  Invalid ppa(%d)", ppa);
		return;
	}

	pci_read_config_dword(znb_pdev[ppa], offset, (uint32_t *)&value);
	return(value);
}

#if 0 /* not needed since we don't use IO access (PDK_BUS_ORDER != 1234) */
uint32_t
pdk_reg_get32(int ppa, uint32_t offset)
{
	/* pdk_htobl is a MACRO, do not combine these lines */
#ifdef PDK_SAL_IO
	uint32_t v = inl(znb_reg[ppa] + offset);
#else
	uint32_t v = readl(znb_reg[ppa] + offset);
#endif

	return (v);
}
#endif


#ifdef BR554
#undef pdk_reg_config_put8
#undef pdk_reg_config_put16
#undef pdk_reg_config_put32
#undef pdk_reg_config_get8
#undef pdk_reg_config_get16
#undef pdk_reg_config_get32
#endif

