/*
**==========================================================================
**
**	./pdk/src/drivers/pal/zxe.c
**
**	Pal Abstraction Layer (PAL)
**
**==========================================================================
**
**	(c) Copyright 1998 ZNYX Corporation
**	All Rights Reserved.
**
**==========================================================================
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License as published by
**  the Free Software Foundation; either version 2 of the License, or
**  (at your option) any later version.
**
**==========================================================================
**
**	Modification History:
**	david	02/16/99	v1.0
**	tim.hayes@znyx.com spring 2K
**	jr	10/01/01	added 8255x support
**      Hamid   4/24/02         use MOD_INC and MOD_DEC outside teh Kernel versi                                on check. Seems like no matter what Kernel we ar				e using we still need to use them
**      Hamid   4/26/02         using iounmap for the clean up  instead of relase_region 
**==========================================================================
**
**	Notes:
**
**==========================================================================
*/

/* new includes for portable ioctl based RMAPI interface */

#include <pdk_port.h>
#include <pdk_global.h>
#include <pdk_types.h>
#include <pdk_constant.h>
#include <pdk_errno.h>
#include <pdk_print.h>
#include <pdk_reg.h>
#include <pdk_mem.h>
#include <pdk_lock.h>
#include <pdk_linux.h>
#include <pdk_lib.h>
#include <pdk_queue.h>
#include <pdk_buf.h>
#include <pdk_time.h>
#include <pdk_hal.h>
#include <pdk_psdb.h>

#include <pdk_pal.h>
#include <pio.h>
#include <rmapi.h>
#include <deviceid.h>
#include <dec21554.h>
#include <dec21040.h>
#include <i82558.h>

#include <linux/config.h>
#include <linux/version.h>
#include <linux/module.h>

#if 0
#ifdef MODULE
#include <linux/module.h>
/*
#ifdef MODVERSIONS
#include <linux/modversions.h>
#endif
#include <linux/module.h>
*/
#else /* 0 */
#if (LINUX_VERSION_CODE >= 0x02031b)
#define NEW_NETINIT
#endif
MOD_INC_USE_COUNT
#define MOD_DEC_USE_COUNT
#endif
#endif /* 0 */

#include <linux/ioport.h>
#include <linux/netdevice.h>
#include <linux/pci.h>
#include <linux/etherdevice.h>
#if (LINUX_VERSION_CODE < 0x020300)
#include <asm/bitops.h>
#include <asm/spinlock.h>
#include <asm/uaccess.h>
#else
#include <linux/spinlock.h>
#endif
#include <dec21040.h>

#ifdef MODULE
int  zxe_debug_arg = 1;
MODULE_PARM(zxe_debug_arg, "i");
#if ZNYX_NB || ZNYX_82559
MODULE_PARM(zxe_media, "1-" __MODULE_STRING(NUM_PORTS) "s");
char zxe_media[NUM_PORTS];
#endif
#endif

/*
 * Function prototypes.
 */
int zxe_open( struct net_device *dev );
int zxe_close( struct net_device *dev );
int zxe_xmit(struct sk_buff *skb, struct net_device *dev);
void zxe_xmit_complete(int32_t ppa, pdk_buf_t *bp, uint32_t status);
void zxe_recv_complete(int32_t ppa, pdk_buf_t *bp, uint32_t status);
void zxe_status(int32_t ppa, uint32_t opcode, void *param);
void zxe_multi_req(struct net_device *dev);
int zxe_set_physaddr_req(struct net_device *dev, void *addr);
struct net_device *zxe_create_netdev(int32_t ppa);
int32_t zxe_znb_num_ports(int32_t subdev);
int32_t zxe_init_tr_bridge(uint32_t ppa);
int32_t zxe_init_nt_bridge(uint32_t ppa);
void zxe_ntbridge_intr(int irq, void *dev_instance, struct pt_regs *regs);
void zxe_intr(int irq, void *dev_instance, struct pt_regs *regs);
void zxe_intr_handler(int ppa);
struct net_device_stats *zxe_getstats(struct net_device *dev);
int zxe_ioctl (struct net_device *dev, struct ifreq *rq, int cmd);
uint32_t pdk_pal_psdb(pdk_psdb_rhdr_t *record);
uint32_t zxe_get_hw_info(int32_t ppa, pal_hw_info_t *psp);

pdk_layer_routines_t my_routines = {
	0, /* pdk_iac */
	0, /* pdk_intr */
	0, /* pdk_xmit */
	0, /* pdk_recv */
	0, /* pdk_ctl */
	zxe_xmit_complete,
	zxe_recv_complete,
	zxe_status,
};
#define PDK_LINUX
#define TMHZDP 

static uint32_t zxe_version = 20317;

#ifdef MODULE
#define VERSION_CONFIG "version 20317: Linux module"
#else
#define VERSION_CONFIG "version 20317: Linux builtin"
#endif

#if ZNYX_ZX231
char zxe_version_string[] = "ZNB zx231 " VERSION_CONFIG;
#endif
#if ZNYX_NB
char zxe_version_string[] = "ZNB NetBlaster " VERSION_CONFIG;
#endif
#if ZNYX_82559
char zxe_version_string[] = "ZNB for 82559 " VERSION_CONFIG;
#endif

/*
 * Set this to nonzero to enable *all* error messages.
 */
int zxe_debug = 1;
extern int sal_debug;
extern int pdk_hal_debug;
extern int pdk_rlk_debug;
#if ZNYX_ZX231
extern uint32_t xchist[];
extern uint32_t no_tcq_handle[];
#endif

extern void freeqinfo(int ppa);
/*
extern void check_bh_lkcnt(int ppa);
*/

#define DEBUG1 if (zxe_debug>0) pdk_print
#define DEBUG2 if (zxe_debug>1) pdk_print
#define DEBUG3 if (zxe_debug>2) pdk_print
#define DEBUG4 if (zxe_debug>3) pdk_print

#if (LINUX_VERSION_CODE >= 0x02031b)
#define NEW_NETINIT
#endif

#define ZXE_DEV_ARRAY(u) ((u)->priv)
#define ZXE_ARRAY_DEV(u) ((u)->za_dev)
#define ZXE_PPA_ARRAY(u) ((u < NUM_PORTS) ? zxe_array[(u)] : 0)

zxe_array_t zxe_array_base[NUM_PORTS];
zxe_array_t *zxe_array[NUM_PORTS];

struct net_device *zrm_dev;

typedef struct pci_id {
	char *name;
	ushort vendor, device;
} pci_id_t;

#define DEV_MASK 0xffff
#define IO_SIZE 0x80
#define RX_LOWATER	0	/* was 4 - never copy */
#define RX_COPY_LEN	1	/* was 100 - never copy */

pci_id_t znb_ids[] = {
#if ZNYX_ZX231
	{"ZX231", 0x110d, 0x002a},
	{"ZX231J",0x110d, 0x002f},	
#endif
#if ZNYX_82559
	{ "i8255x", 0x8086, 0x1229 },	/* 82557, 82558, 82559	*/
	{ "i82559ER", 0x8086, 0x1209 },	/* 82559ER		*/

#if 0
	{ 0x8086, 0x1029, PCI_ANY_ID, PCI_ANY_ID, },
	{ 0x8086, 0x1031, PCI_ANY_ID, PCI_ANY_ID, },
	{ 0x8086, 0x1032, PCI_ANY_ID, PCI_ANY_ID, },
	{ 0x8086, 0x1033, PCI_ANY_ID, PCI_ANY_ID, },
	{ 0x8086, 0x1034, PCI_ANY_ID, PCI_ANY_ID, },
	{ 0x8086, 0x1038, PCI_ANY_ID, PCI_ANY_ID, },
	{ 0x8086, 0x1209, PCI_ANY_ID, PCI_ANY_ID, },
	{ 0x8086, 0x1229, PCI_ANY_ID, PCI_ANY_ID, },
	{ 0x8086, 0x2449, PCI_ANY_ID, PCI_ANY_ID, },
#endif
#endif
#if ZNYX_NB
	{ "ZNYX", 0x1011, 0x0009},
	{ "ZNYX", 0x1011, 0x0019},
	{ "ZNYX", 0x1011, 0x0046},
	{ "ZNYX", 0x8086, 0xB555},
#endif
	{0},
};

typedef struct pci_sub_id {
	char *name;
	ushort vendor, device;
} pci_sub_id_t;

pci_sub_id_t znb_sub_ids[] = {
	{ "Unknown", 0, 0},
	{ "ZNYX ZNB345Q", 0x110D, 0x0012},
	{ "ZNYX ZNB346Q", 0x110D, 0x0013},
	{ "ZNYX ZNB348Q", 0x110D, 0x0014},
	{ "ZNYX ZNB414", 0x110d, 0x0018},
	{ "ZNYX ZNB412", 0x110d, 0x0019},
	{ "ZNYX 554", 0x110d, 0x001b},
	{ "ZNYX Q-Card", 0x110d, 0},
	{0},
};

pdk_hal_stat_t hal_stat_storage;
struct net_device_stats zxe_stats[NUM_PORTS];

/*
 * Ethernet broadcast address definition.
 */
static struct pdk_ether_addr etherbroadcastaddr = {
	{0xff, 0xff, 0xff, 0xff, 0xff, 0xff}
};

#define ZXE_MCA(eaddr) (((caddr_t)(eaddr))[0] & 1)
#define	ZXE_NUM_MCAST	14
#define ZXE_OPEN		0x01

boolean znb_hwstart(int32_t ppa);

#define ZXE_HAL_RESET 0
#define ZXE_HAL_INITTED 1
#define ZXE_HAL_STARTED 2

int znb_initted = 0, znb_recently_initted = 0;

#define	ether_copy(a, b) (pdk_bcopy((caddr_t)a, (caddr_t)b, PDK_ETHERADDRL))
#define	ether_cmp(a, b) (pdk_bcmp((caddr_t)a, (caddr_t)b, PDK_ETHERADDRL))

#define BIOS_SHIFT 8

static void pktdump(skb)
struct sk_buff *skb;
{
	uchar *ch = (uchar *)(skb->data);

	DEBUG1("   : %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x\n",
		ch[0],ch[1],ch[2],ch[3],ch[4],ch[5],ch[6],ch[7],ch[8],ch[9],
		ch[10],ch[11],ch[12],ch[13],ch[14],ch[15]);
}

unsigned long splimp(void);
unsigned long splimp(void)
{
	unsigned long s;

	save_flags(s);
	cli();
	return(s);
}
void splx(unsigned long s)
{
	restore_flags(s);
}


struct net_device *zxe_create_netdev(int32_t ppa)
{
	struct net_device *dev;
	zxe_array_t *zat;

	DEBUG3("zxe_create_netdev(%d): enter\n", ppa);

	/* Initialize the Hardware Abstraction Layer */
	zat = &zxe_array_base[ppa];
	zxe_array[ppa]=zat;
	zat->za_ppa = ppa;
	zat->za_state = -1;  /* otherwise not set until INITed below */
	zat->za_nolink = 1;  /* avoid xmit when ifconfig UP but no link yet */

	pdk_lock_init(ppa);
	pdk_mem_init(ppa);
	pdk_time_init(ppa);
	pdk_buf_init(ppa);
	pdk_layer_init(ppa);

	if (pdk_hal_init(ppa) != 0) {
		DEBUG3("zxe_create_netdev(%d): error\n", ppa);
		return(NULL);
	}

	dev = init_etherdev(NULL, 0);
	zat->za_dev = dev;
	sprintf(dev->name,"%s%d","znb",ppa);

	zat->za_state = ZXE_HAL_INITTED;
	if (pdk_layer_push(ppa, PDK_HAL_LAYER) != 0) {
		DEBUG1("zxe_create_netdev %d: pdk_layer_push failed\n", ppa);
		return(NULL);
	}

	if (pdk_layer_push(ppa, PDK_PAL_LAYER) != 0) {
		DEBUG1("zxe_create_netdev %d: pdk_layer_push failed\n", ppa);
		return(NULL);
	}

	(*zat->za_llr.pdk_ctl)(ppa, PDK_HAL_GET_PHYSADDR, (void *) &zat->za_curaddr);

	ether_copy(&zat->za_curaddr, &dev->dev_addr);
	dev->base_addr = znb_reg[ppa];
	dev->open = &zxe_open;
	dev->stop = &zxe_close;
	dev->hard_start_xmit = &zxe_xmit;
	dev->get_stats = &zxe_getstats;
	dev->do_ioctl = &zxe_ioctl;
	dev->set_multicast_list = &zxe_multi_req;
	dev->set_mac_address = &zxe_set_physaddr_req;
	dev->priv = zat;
	DEBUG3("zxe_create_netdev(%d): done\n", ppa);
	return(dev);
}

int32_t
zxe_znb_num_ports(int32_t subdev)
{
	int32_t num_ports=-1;
	int32_t ntbridge_device=0;
	switch(subdev) {
		case SUBDEVICEID(ADAPTER_ZX474):
			num_ports = 4;
			ntbridge_device=1;
			DEBUG1("zxe_znb_num_ports discovered ZX474\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX476):
			num_ports = 6;
			ntbridge_device=1;
			DEBUG1("zxe_znb_num_ports discovered ZX476\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX478):
			num_ports = 8; 
			ntbridge_device=1;
			DEBUG1("zxe_znb_num_ports discovered ZX478\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX444):
			num_ports = 4;
			ntbridge_device=1;
			DEBUG1("zxe_znb_num_ports discovered ZX444\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX442):
			num_ports = 2;
			ntbridge_device=1;
			DEBUG1("zxe_znb_num_ports discovered ZX442\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX412):
			DEBUG1("zxe_znb_num_ports discovered ZX412\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX414):
			DEBUG1("zxe_znb_num_ports discovered ZX414\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX214):
			DEBUG1("zxe_znb_num_ports discovered ZX214\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX212):
			DEBUG1("zxe_znb_num_ports discovered ZX212\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX345):
			DEBUG1("zxe_znb_num_ports discovered ZX345\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX346):
			DEBUG1("zxe_znb_num_ports discovered ZX346\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX348):
			DEBUG1("zxe_znb_num_ports discovered ZX348\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX345Q):
			DEBUG1("zxe_znb_num_ports discovered ZX345Q\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX348Q):
			DEBUG1("zxe_znb_num_ports discovered ZX348Q\n");
			break;
		case SUBDEVICEID(ADAPTER_ZX346Q):
			DEBUG1("zxe_znb_num_ports discovered ZX346Q\n");
			break;
		default:
			DEBUG1("zxe_znb_num_ports discovered generic adapter\n");
			num_ports = 1;
			ntbridge_device=0;
			break;
	}
	return(num_ports);
}

int32_t 
zxe_init_nt_bridge(uint32_t ppa)
{
	uint32_t mask, base_addr;
	struct pci_dev *pdev = znb_pdev[ppa];
	int32_t num_ppa = 0;
	ushort pci_cmd, required_cmd, subsys_dev;
	struct net_device *dev;
	ulong start;
	ulong size;
	int irq;
	int i;
	int32_t base_ppa;
	zxe_array_t *zat;
	int ret=0;

#if (LINUX_VERSION_CODE < 0x02030d)
	start = pdev->base_address[3];
#else
	start = pdev->resource[3].start;
#endif
	pdk_reg_config_put32(ppa, PCI_BASE_ADDRESS_3, ~0);
	mask = pdk_reg_config_get32(ppa, PCI_BASE_ADDRESS_3); 
	size = ~mask+1;
	start = start & PCI_BASE_ADDRESS_MEM_MASK;
	pdk_reg_config_put32(ppa, PCI_BASE_ADDRESS_3, start);
	base_addr = (int32_t) ioremap_nocache(PDK_PCI_MEM_BASE + start,size);
	znb_reg[ppa] = base_addr;
	irq = pdev->irq;
	pdk_reg_config_put32(ppa, PCI_COMMAND,0x116);
	if (pdk_ntbridge_init(ppa) != 0) {
		DEBUG1("zxe_init_nt_bridge %d: pdk_ntbridge_init failed\n", ppa);
		return(0);
	}

#if (LINUX_VERSION_CODE < 0x020300)
	pcibios_read_config_word(pdev->bus->number, pdev->devfn, PCI_SUBSYSTEM_ID, &subsys_dev);
	num_ppa = zxe_znb_num_ports(subsys_dev);
#else
	num_ppa = zxe_znb_num_ports(pdev->subsystem_device);
#endif

	base_ppa=ppa;
	for (i=0;i<num_ppa;i++){
/* fixme: window for any chip */
		znb_reg[ppa] = base_addr + i*BR554_21143_WINDOW;
		if(pdk_ntbridge_find_and_init_device(base_ppa, DEC_21143_PCI_ID, ppa) == 0)  {
			DEBUG1("zxe_init_nt_bridge(%d) found ppa %d\n", base_ppa, ppa);
		} else {
			DEBUG1("zxe_init_nt_bridge(%d) failed for ppa %d\n", base_ppa, ppa);
			/* just fail, but could return number of OK ppas */
			return(0);
		}
		pci_cmd = pdk_reg_config_get16(ppa, PCI_COMMAND);
		required_cmd = pci_cmd | PCI_COMMAND_MASTER | PCI_COMMAND_MEMORY | PCI_COMMAND_INVALIDATE;

		pdk_reg_config_put16(ppa, PCI_COMMAND, required_cmd);
		dev = zxe_create_netdev(ppa);
		if (dev==NULL) {
			DEBUG1("Null device in zxe_init_nt_bridge\n");
			return(0);
		}
		znb_pdev[ppa] = pdev;
		dev->irq=irq;
		/* Need zxe_intr() wrapper since ntbridge code supplies one arg. */
		pdk_ntbridge_register(ppa, zxe_intr_handler, (void *)ppa);
		/* Since irq doesn't change, and zxe_ntbridge_intr walks queue
		 * of registered ppas, call request_irq() only once per bridge.
		 */
		if (ppa == base_ppa) {
			if ((ret=request_irq(dev->irq, &zxe_ntbridge_intr, SA_SHIRQ, dev->name, dev))) {
				DEBUG1("zxe_init_nt_bridge %d: request_irq failed irq: 0x%x ret: 0x%x\n",
				ppa, dev->irq, ret);
				return(0);
			}
		}
		ppa++;
	}
	return(num_ppa);
}

int32_t 
zxe_init_tr_bridge(uint32_t ppa)
{
	uint32_t mask, base_addr;
	struct pci_dev *pdev = znb_pdev[ppa];
	struct net_device *dev;
	int irq;
	int ret=0;
	ulong start;
	ulong size;

	DEBUG2("zxe_init_tr_bridge %d: id=%08x %08x %08x sid=%08x pm=%08x\n", ppa,
		pdk_reg_config_get32(ppa, 0),
		pdk_reg_config_get32(ppa, 4),
		pdk_reg_config_get32(ppa, 8),
		pdk_reg_config_get32(ppa, 0x2c),
		pdk_reg_config_get32(ppa, 0xe0),
		0);
#if ZNYX_ZX231
#if (LINUX_VERSION_CODE < 0x02030d)
	start = pdev->base_address[0];
#else
	start = pdev->resource[0].start;
#endif
	pdk_reg_config_put32(ppa,PCI_COMMAND,0x110);
	pdk_reg_config_put32(ppa, PCI_BASE_ADDRESS_0, ~0);
	mask = pdk_reg_config_get32(ppa, PCI_BASE_ADDRESS_0); 
	start = start & PCI_BASE_ADDRESS_MEM_MASK;
	pdk_reg_config_put32(ppa, PCI_BASE_ADDRESS_0, start);
#endif /* ZNYX_ZX231 */

#if ZNYX_82559
#if (LINUX_VERSION_CODE < 0x02030d)
	start = pdev->base_address[0];
#else
	start = pdev->resource[0].start;
#endif
	pdk_reg_config_put32(ppa, PCI_BASE_ADDRESS_0, ~0);
	mask = pdk_reg_config_get32(ppa, PCI_BASE_ADDRESS_0); 
	start = start & PCI_BASE_ADDRESS_MEM_MASK;
	pdk_reg_config_put32(ppa, PCI_BASE_ADDRESS_0, start);
#endif /* ZNYX_82559 */

#if ZNYX_NB
#if (LINUX_VERSION_CODE < 0x02030d)
	start = pdev->base_address[1];
#else
	start = pdev->resource[1].start;
#endif
	pdk_reg_config_put32(ppa, PCI_BASE_ADDRESS_1, ~0);
	mask = pdk_reg_config_get32(ppa, PCI_BASE_ADDRESS_1); 
	start = start & PCI_BASE_ADDRESS_MEM_MASK;
	pdk_reg_config_put32(ppa, PCI_BASE_ADDRESS_1, start);
#endif /* ZNYX_NB */

	size = ~mask+1;
	base_addr = (int32_t) ioremap_nocache(PDK_PCI_MEM_BASE + start, size);
	znb_reg[ppa] = base_addr;
	DEBUG2("zxe_init_tr_bridge %d: start=%x size=%x base=%x\n", ppa, start, size, base_addr);
	irq = pdev->irq;
	pdk_reg_config_put32(ppa, PCI_COMMAND,0x116);

	dev=zxe_create_netdev(ppa);
	if (dev == 0) {
		release_region(base_addr, IO_SIZE);
		znb_reg[ppa] = 0;
		return(0);
	}

	dev->irq=irq;
	if ((ret=request_irq(dev->irq, &zxe_intr, SA_SHIRQ, dev->name, dev))) {
		DEBUG1("zxe_init_tr_bridge %d: request_irq failed irq: 0x%x ret: 0x%x\n",
		ppa, dev->irq, ret);
		return(0);
	}
	return(1);
}

static int probed = 0;

/* TMHXXX: Modify here for module support */
/* Huh?  I see 2.2.x vs 2.3.x */
#ifdef NEW_NETINIT
int zxe_probe(void)
#else 
int zxe_probe(struct net_device *dev)
#endif
{
	zxe_array_t *zat;
	struct pci_dev *pdev = NULL;
	static int ppa;
	ushort subsys_vendor;
	int i;

	DEBUG3("zxe_probe(%d): enter\n", ppa);

	/* Not needed for MODULE since probe() called from init_module().
	 * Not needed (apparently) in 2.3.x builtin?
	 * Needed in 2.2.x builtin. Why does Linux make multiple probe() calls?
	 */

	if (probed) return -ENODEV;
	probed++;

	ppa = 0;
	
	if ( !pcibios_present()) {
		return -ENODEV;
	}

	DEBUG1("zxe_probe: %s\n", zxe_version_string);

#ifndef MODULE
	init_module();
#endif

	for(i=0, pdev=NULL; znb_ids[i].name != NULL; i++) {
		DEBUG1("zxe_probe: looking for %s %x\n",znb_ids[i].name,znb_ids[i].device);
		while ((pdev = pci_find_device(znb_ids[i].vendor,znb_ids[i].device, pdev)) != NULL) {

#if ZNYX_NB
#if (LINUX_VERSION_CODE < 0x020300)
			pcibios_read_config_word(pdev->bus->number, pdev->devfn,
				PCI_SUBSYSTEM_VENDOR_ID, &subsys_vendor);
			if (subsys_vendor != SUBVENDORID_ZNYX)
#else
			if (pdev->subsystem_vendor != SUBVENDORID_ZNYX)
#endif
			{
				continue;
				/*if (pdev->device != 0x9)  Was this for 21140?
				 *	continue;
				 */
			}
#endif

			znb_pdev[ppa] = pdev;
#if ZNYX_ZX231
			switch(pdev->device) {
				case SUBDEVICEID(ADAPTER_ZX231):
					DEBUG1("znb_probe[%d]: found ZX231 %s\n", ppa,znb_ids[i].name);
					ppa += zxe_init_tr_bridge(ppa);
					break;

				case SUBDEVICEID(ADAPTER_ZX231J):
					DEBUG1("znb_probe[%d]: found ZX231J %s\n", ppa,znb_ids[i].name);
					ppa += zxe_init_tr_bridge(ppa);
					break;

				default:
					DEBUG1("znb_probe[%d]: Adapter type 0x%x not supported\n", pdev->device);
					break;
			}
#endif
#if ZNYX_NB || ZNYX_82559

		 	if ((pdev->device == (DEC_21554_PCI_ID>>16))||(pdev->device == (DEC_21555_PCI_ID>>16) ) ) 
			{
				DEBUG1("znb_probe[%d]: found Non Transparent Bridge %s\n", ppa,znb_ids[i].name);
				ppa += zxe_init_nt_bridge(ppa);
			}
			else {
				DEBUG1("znb_probe[%d]: found NIC %s\n", ppa,znb_ids[i].name);
				ppa += zxe_init_tr_bridge(ppa);
			}
#endif
		}
	}

	if ( ppa != 0 ) {
		/* allocate zrm device for RMAPI */
		zrm_dev = init_etherdev(NULL, 0);
		sprintf(zrm_dev->name,"zrm");

		zrm_dev->open = &zxe_open;
		zrm_dev->stop = &zxe_close;
		zrm_dev->do_ioctl = &zxe_ioctl;
		zrm_dev->priv = &zrm_dev;
	}

	DEBUG1("znb_probe: found %d ppa's, returning %d\n",
		ppa, (ppa != 0) ? 0 : -ENODEV );

	/* if we never made it through the loop, ppa == 0 */
	return ( (ppa != 0) ? 0 : -ENODEV );
}


int
init_module(void)
{

	/* Initialize the System Abstraction Layer */
	if (znb_initted == 0) {
		znb_recently_initted++;
		pdk_lock_global_init();
		pdk_mem_global_init();
		pdk_time_global_init();
		pdk_buf_global_init();
		pdk_layer_global_init();
#ifdef TMHXXX
/* HUH?   It zeros znb_pdev, which isn't set until zxe_probe. */
/* this reinits znb_reg which is already inited - reinit breaks it */
		pdk_reg_global_init();
#endif
		pdk_bzero(zxe_array, sizeof(zxe_array));
		pdk_hal_global_init();
		znb_initted++;
	}
#if MODULE
#ifdef NEW_NETINIT
	return(zxe_probe());
#else
	return(zxe_probe(NULL));
#endif
#else
	return(0);
#endif
}

/* TODO  doesn't cleanup for 554 correctly.  See mmc's code. */

void
cleanup_module(void)
{
	zxe_array_t *zat;
	struct pci_dev *pdev;
	int i, ppa;

	DEBUG1("znb cleanup\n");

	pdk_layer_global_deinit();
	pdk_time_global_deinit();
	pdk_buf_global_deinit();
	pdk_lock_global_deinit();
	pdk_mem_global_deinit();

	/* undo what was done in zxe_probe() */
/* Count down so that if it's a ntbridge, free_irq() is called
 * with the same *dev as request_irq().
 */
	for (i = NUM_PORTS-1; i >= 0; i--) {
		if ((zat = zxe_array[i]) == 0)
			continue;
		/* All inits done at zxe_probe() time. If znb was insmod
		 * and rmmod without an ifconfig (i.e. zxe_open)
		 * then deiniting is done here since zxe_close wasn't called).
		 */
		ppa = zat->za_ppa;
		if (zat->za_state == ZXE_HAL_INITTED) {
			/* Deinitialize the Hardware Abstraction Layer */
			pdk_hal_deinit(ppa);
			zat->za_state = ZXE_HAL_RESET;
				
			zat->za_flags &= ~ZXE_OPEN;

			pdk_layer_deinit(ppa);
			pdk_time_deinit(ppa);
			pdk_buf_deinit(ppa);
			pdk_lock_deinit(ppa);
			pdk_mem_deinit(ppa);
		}

		pdev = znb_pdev[ppa];
		if (pdev->device == (DEC_21554_PCI_ID>>16) ) {
			pdk_ntbridge_unregister(ppa);
			if (!pdk_ntbridge_device_deinit(ppa)) {
				free_irq(zat->za_dev->irq, zat->za_dev);
			}
		}
		else {
			free_irq(zat->za_dev->irq, zat->za_dev);
		}
		unregister_netdev(zat->za_dev);
               /* commnted the release_region and use iounmap instaed (gh)*/
/*		release_region(zat->za_dev->base_addr, IO_SIZE);*/
		iounmap(zat->za_dev->base_addr);
DEBUG1("za_dev=%x\n", zat->za_dev);
		pdk_mem_free(ppa, (uchar *)zat->za_dev, sizeof(*zat->za_dev));
		zxe_array[i] = 0;
	}

	if (zrm_dev) {
		unregister_netdev(zrm_dev);
	}
}

pdk_layer_routines_t *
pdk_pal_register(int ppa, pdk_layer_routines_t *up, pdk_layer_routines_t *down)
{
	zxe_array_t *zat;
	pdk_layer_routines_t *lrp;

	DEBUG2("pdk_pal_register(%d):  Registering up 0x%x down 0x%x\n", ppa, up, down);
	if ((zat = zxe_array[ppa]) == 0) {
		DEBUG1("pdk_pal_register(%d): zat not initialized\n", ppa);
		return(0);
	}

	lrp = &zat->za_llr;

	if (up) {
		DEBUG1("pdk_pal_register(%d):  No layers should be above the PAL\n", ppa);
		return(0);
	}

	if (down) {
		lrp->pdk_iac = down->pdk_iac;
		lrp->pdk_intr = down->pdk_intr;
		lrp->pdk_xmit = down->pdk_xmit;
		lrp->pdk_recv = down->pdk_recv;
		lrp->pdk_ctl = down->pdk_ctl;
	}

	return(&my_routines);
}

struct net_device_stats *
zxe_getstats(struct net_device *dev)
{
	zxe_array_t *zat;
	struct net_device_stats *zsp;

	DEBUG3("zxe_getstats: enter\n");

	zat = ZXE_DEV_ARRAY(dev);
	if (zat == 0)
		return(0);

	zsp = &zxe_stats[zat->za_ppa];

	pdk_bzero(&hal_stat_storage, sizeof(hal_stat_storage));

	if ((*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_GET_COUNTERS,
		(void *) &hal_stat_storage) != 0) {
		return(0);
	}

/* TMHXXX get this from hal stats */

	zsp->rx_missed_errors = 0;
	zsp->rx_packets = hal_stat_storage.hs_rx_packets;
	zsp->tx_packets = hal_stat_storage.hs_tx_packets;
	/* zsp->rx_bytes = handled in recv_complete */
#if LINUX_VERSION_CODE > 0x20127
	/* added hs_tx_bytes */
	zsp->tx_bytes = hal_stat_storage.hs_tx_bytes;
/* WHY not have this? */
#ifdef TMHXXX
DEBUG1("txb=%d vc=0x%08x\n", hal_stat_storage.hs_tx_bytes, LINUX_VERSION_CODE);
#endif
#endif
	zsp->rx_errors = hal_stat_storage.hs_rx_err_packets;
	zsp->tx_errors = hal_stat_storage.hs_tx_err_packets;
	zsp->collisions = hal_stat_storage.hs_tx_one_collision
		+ hal_stat_storage.hs_tx_more_collisions;

	/* detailed rx_errors: */
	zsp->rx_length_errors = hal_stat_storage.hs_rx_frame_too_long;
	zsp->rx_crc_errors =
		hal_stat_storage.hs_rx_crc + hal_stat_storage.hs_rx_mii_error;
	zsp->rx_frame_errors = hal_stat_storage.hs_rx_alignment_error;
	zsp->rx_fifo_errors = hal_stat_storage.hs_rx_overrun;

	/* detailed tx_errors */
	/* Is link fail be part of aborted?  Hal doesn't look at TDES0_LF. */
	zsp->tx_aborted_errors = hal_stat_storage.hs_tx_jabber + hal_stat_storage.hs_tx_max_collisions;
	zsp->tx_carrier_errors = hal_stat_storage.hs_tx_carrier_loss + hal_stat_storage.hs_tx_no_carrier;
	zsp->tx_fifo_errors = hal_stat_storage.hs_tx_underrun;

	/* TDES0_LC:  why "window" error? */
	zsp->tx_window_errors = hal_stat_storage.hs_tx_late_collisions;

	DEBUG3("zxe_getstats: done\n");

	return (zsp);
}


uint32_t zxe_eaddr(int32_t trunk_ppa);
uint32_t zxe_flags(int32_t trunk_ppa);

uint32_t
zxe_eaddr(int32_t trunk_ppa)
{
	zxe_array_t *zat;
	struct net_device *dev;

	zat = ZXE_PPA_ARRAY(trunk_ppa);
	if (zat == 0)
		return(0);

	dev = ZXE_ARRAY_DEV(zat);

	ether_copy(&zat->za_curaddr, &dev->dev_addr);
	return(0);
}

uint32_t
zxe_destroy_if(ppa)
int32_t ppa;
{
	zxe_array_t *zat;
	struct net_device *dev;

	DEBUG3("zxe_destroy(%d): enter\n", ppa);

	zat = ZXE_PPA_ARRAY(ppa);
	if (zat == 0)
		return(0);

	dev = ZXE_ARRAY_DEV(zat);

	unregister_netdevice(dev);
	DEBUG3("zxe_destroy(%d): done\n", ppa);
}

uint32_t
zxe_create_if(ppa, if_name)
int32_t ppa;
char *if_name;
{
	zxe_array_t *zat;
	struct net_device *dev;

	DEBUG3("zxe_create_if(%d): enter\n", ppa);

	zat = ZXE_PPA_ARRAY(ppa);
	if (zat == 0)
		return(0);

	dev = ZXE_ARRAY_DEV(zat);

	sprintf(dev->name,"%s", if_name);

	register_netdevice(dev);
	DEBUG3("zxe_create_if(%d): done\n", ppa);
	return(0);
}

uint32_t
zxe_flags(int32_t ppa)
{
	zxe_array_t *zat;
	struct net_device *dev;

	DEBUG3("zxe_flags(%d): called\n", ppa);

	zat = ZXE_PPA_ARRAY(ppa);
	if (zat == 0)
		return(0);

	dev = ZXE_ARRAY_DEV(zat);

	if ( dev->flags & (IFF_UP | IFF_RUNNING) ) {
		return(1);
	} else {
		return(0);
	}
}

uint32_t
pdk_pal_ctl(ppa, opcode, param)
int32_t ppa;
uint32_t opcode;
void *param;
{
	uint32_t ret;
	char if_name[8];

	ret = 0;

	switch(opcode) {
		case PDK_PSDB_INLINE:
			pdk_pal_psdb((pdk_psdb_rhdr_t *) param);
			break;
		case PDK_PAL_CREATE_ZNB_IF:
			sprintf(if_name, "znb%d", (int32_t) param);
			zxe_create_if(ppa, if_name);
			break;
		case PDK_PAL_CREATE_ZRL_IF:
			sprintf(if_name, "zrl%d", (int32_t) param);
			zxe_create_if(ppa, if_name);
			break;
		case PDK_PAL_DESTROY_ZNB_IF:
		case PDK_PAL_DESTROY_ZRL_IF:
			zxe_destroy_if(ppa);
			break;
		case PDK_PAL_SET_PHYSADDR:
			ret = zxe_eaddr(ppa);
			break;
		case PDK_PAL_GET_PHYSADDR:
			break;
		case PDK_PAL_PORT_CONFIGURED: {
			zxe_array_t *zat = zxe_array[ppa];

			if (zat == 0)
				return(0);

			ret = zxe_flags(ppa);
			break;
		}
		case PDK_PAL_PORT_EXISTS: {
			zxe_array_t *zat = zxe_array[ppa];

			if (zat)
				ret=1;
			break;
		}

	}

	return(ret);
}

uint32_t
pdk_pal_psdb(record)
pdk_psdb_rhdr_t *record;
{
	uint32_t record_type, record_size;
	uint32_t psdb_num_ports;
	uint32_t *lp;
	int32_t ppa;
	int32_t i, err;

	record_type = psdb_pton(record->ppr_header.Type);
	record_size = psdb_pton(record->ppr_header.Size);

	lp = (uint32_t *) record;
	for (i=0;i<(int32_t)(record_size/sizeof(uint32_t));i++)
		lp[i] = psdb_pton(lp[i]);

	ppa = (int32_t) record->ppr_ppa;

	err = 0;

	switch(record_type) {
		case PSDB_TYPE_PAL_GET_HW_INFO: {
			pdk_psdb_pal_get_hw_info_t *ppsp;
			uint32_t *lp, *record_lim;
			uint32_t tmp;

			if (record_size != sizeof(pdk_psdb_pal_get_hw_info_t)) {
				return(PSDB_INCORRECT_SIZE);
			}

			ppsp = (pdk_psdb_pal_get_hw_info_t *) record;

			err = zxe_get_hw_info(ppa, &ppsp->pps_info);

			if (err)
				return(err);

			lp = (uint32_t *) &ppsp->pps_info;
			record_lim = (uint32_t *)((char *)record + record_size); 
			while (lp < record_lim) {
				tmp =*lp;
				*lp++ = psdb_ntop(tmp);
			}

			break;
		}
		default:
			err = PSDB_BAD_PARAM;
			break;
	}

	return(err);
}

uint32_t
zxe_get_hw_info(ppa, psp)
int32_t ppa;
pal_hw_info_t *psp;
{
	zxe_array_t *zat;
	uchar	fa[6],ca[6];

	DEBUG3("zxe_get_hw_info(%d): called\n", ppa);

	pdk_bzero(psp, sizeof(pal_hw_info_t));

	psp->hi_version = HI_VERSION;
	psp->hi_sft_version = zxe_version;

	zat = ZXE_PPA_ARRAY(ppa);
	if (zat == 0) {
		psp->hi_state = HI_STATE_NOT_INITED;
		return(0);
	}

	if (zat->za_state != ZXE_HAL_STARTED) {
		psp->hi_state = HI_STATE_NOT_INITED;
		return(0);
	}

	psp->hi_state = HI_STATE_EXISTS;
	/* this is not a version thing, it's 554 or NOT_554 */
	/* DJ:  for NOT_554, this info is in znb_pdev[ppa] */
#ifdef TMHXXX 
bus and fcn went away with 554 
	psp->hi_bus = znb_reg[ppa].bus;
	psp->hi_devno = PCI_SLOT(znb_reg[ppa].fcn);
#endif
	pdk_hal_ctl(ppa, PDK_HAL_GETID, (void *)&psp->hi_adapter_id);

	psp->hi_chipid = pdk_reg_config_get32(ppa, PCI_VENDOR_ID);
	psp->hi_revinfo = pdk_reg_config_get32(ppa, PCI_REVISION_ID);
	psp->hi_cfgcs = pdk_reg_config_get32(ppa, PCI_COMMAND);
	psp->hi_ltcls = pdk_reg_config_get32(ppa, CFLT);
#if ZNYX_ZX231 || ZNYX_82559
	psp->hi_bar[0] = pdk_reg_config_get32(ppa, PCI_BASE_ADDRESS_0);
#endif
#if ZNYX_NB
	psp->hi_bar[0] = pdk_reg_config_get32(ppa, PCI_BASE_ADDRESS_1); 
#endif
	psp->hi_ssid = pdk_reg_config_get32(ppa, PCI_SUBSYSTEM_VENDOR_ID);
	psp->hi_rombar = pdk_reg_config_get32(ppa, PCI_ROM_ADDRESS);
	psp->hi_irq = pdk_reg_config_get32(ppa, PCI_INTERRUPT_LINE);

	pdk_hal_ctl(ppa, PDK_HAL_GET_PHYSADDR, (void *) ca);
	pdk_hal_ctl(ppa, PDK_HAL_GET_HDWRADDR, (void *) fa);

	psp->hi_macaddr[0] = (fa[0]<<24) |  (fa[1]<<16) |  (fa[2]<<8) |  fa[3];
	psp->hi_macaddr[1] = (fa[4]<<24) |  (fa[5]<<16) |  (ca[0]<<8) |  ca[1];
	psp->hi_macaddr[2] = (ca[2]<<24) |  (ca[3]<<16) |  (ca[4]<<8) |  ca[5];

	return(0);
}


int 
zxe_ioctl(struct net_device *dev, struct ifreq *ifr, int cmd)
{
	zxe_array_t *zat = (zxe_array_t *)(dev->priv);
	unsigned long s;
	ioctl_data_t *piodata;
	int status = 0, type = 0;

	DEBUG3("zxe_ioctl: enter\n");
/*
	s = splimp();
*/

	switch (cmd) {
		case SIOCDEVPRIVATE+5:
			(*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_GET_COUNTERS,
				(void *) ifr->ifr_data);
			break;
		case SIOCDEVPRIVATE+6:
			pdk_hal_debug = 2;
			(*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_GET_COUNTERS,
				(void *) ifr->ifr_data);
			pdk_hal_debug = 1;
			break;
#if ZNYX_ZX231
		/* Extra zx231 debug */
		case SIOCDEVPRIVATE+7:
			hal_mac_counts(zat->za_ppa);
			break;
		case SIOCDEVPRIVATE+8:
			hal_clear_counts(zat->za_ppa);
			break;
		case SIOCDEVPRIVATE+9:
			pdk_bcopy((char *)(&xchist[1]), (char *)ifr->ifr_data, 96*sizeof(uint32_t));
			pdk_bcopy((char *)(&no_tcq_handle[1]), &((char *)(ifr->ifr_data))[96*sizeof(uint32_t)], 96*sizeof(uint32_t));
			break;
#endif
		case SIOZOPEN:
		case SIOZBLOPEN:
		case SIOZCLOSE:
		case SIOZREAD:
		case SIOZWRITE:
			piodata= (ioctl_data_t *)ifr->ifr_data;

			if (pio_ioctl(cmd, piodata) != 0)
				status = -EOPNOTSUPP;
			break;

		case SIOCSIFMAP:
			switch (ifr->ifr_map.port) {
				case IF_PORT_10BASE2:
					type = ZXE_MEDIA_BNC;
					break;
				case IF_PORT_10BASET:
					type = ZXE_MEDIA_TP;
					break;
				case IF_PORT_AUI:
					type = ZXE_MEDIA_AUI;
					break;
				case IF_PORT_100BASET:
					type = ZXE_MEDIA_T4;
					break;
				case IF_PORT_100BASETX:
					type = ZXE_MEDIA_TX;
					break;
				default:
					DEBUG1("zxe_ioctl(%d) unknown mediatype: %d\n", zat->za_ppa, type);
					status = -EOPNOTSUPP;
			}
				
			(*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_SET_MEDIATYPE, (void *) type);

		default:
			DEBUG2("zxe_ioctl(%d) unknown command: %d\n", zat->za_ppa, cmd);
			status = -EOPNOTSUPP;
	}
	
/*
	splx(s);
*/
	DEBUG3("zxe_ioctl: done\n");

	return(status);
}


boolean
znb_hwstart(ppa)
int32_t ppa;
{
	zxe_array_t *zat;

	DEBUG3("zxe_hwstart(%d): enter\n", ppa);

	zat = ZXE_PPA_ARRAY(ppa);
	if (zat == 0) {
		DEBUG3("zxe_hwstart(%d): error\n", ppa);
		return(0);
	}
	DEBUG2("znb_hwstart:  Starting interface %d\n", ppa);

	if ((*zat->za_llr.pdk_ctl)(ppa, PDK_HAL_INIT_AND_START, 0) != 0) {
		DEBUG1("znb_hwstart(%d):  PDK_HAL_START failed\n", ppa);
		return (1);
	}

	if ((*zat->za_llr.pdk_ctl)(ppa, PDK_HAL_RECEIVE_INIT, 0) != 0) {
		DEBUG1("znb_hwstart(%d):  PDK_HAL_RECEIVE_INIT failed\n", ppa);
		(*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_STOP_AND_RESET, 0);
		return (1);
	}

	zat->za_state = ZXE_HAL_STARTED;

	DEBUG3("zxe_hwstart(%d): done\n", ppa);
	return(0);
}

spinlock_t znb_lock;

#if ZNYX_NB || ZNYX_82559
#ifdef MODULE
typedef struct {
	char name[16];
	int  type;
} zxe_media_t;

zxe_media_t zxe_medianame[] = {
	"auto",	ZXE_MEDIA_AUTO,
	"tp",	ZXE_MEDIA_TP,
	"tpfd",	ZXE_MEDIA_TPFD,
	"tx",	ZXE_MEDIA_TX,
	"txfd",	ZXE_MEDIA_TXFD
	};

/* TODO ?? "char zxe_media[16]" is being used as a "char *zxe_media[16]".
 * Don't know why.  Not behaving as expected (drivers/net/cs89x0.c example).
 */
char **zmedia = (char **)zxe_media;

#endif /* MODULE */
#endif

int
zxe_open(struct net_device *dev)
{
	zxe_array_t *zat;
	int32_t ppa, i, forcemedia = 0;

	DEBUG1("zxe_open: %s dev 0x%x\n", dev->name, dev);

	znb_lock = (spinlock_t)SPIN_LOCK_UNLOCKED;

	if (dev == zrm_dev) {
		DEBUG3("zxe_open: zrm_dev\n");
		return(-ENODEV);
	}

	zat = ZXE_DEV_ARRAY(dev);
	if (zat == 0) {
		DEBUG3("zxe_open: no zat\n");
		return(-ENODEV);
	}

	ppa = zat->za_ppa;

	if (zat->za_flags & ZXE_OPEN || zat->za_state == ZXE_HAL_STARTED) {
		DEBUG1("zxe_open(%d): already in use.\n", ppa);
		return(-EBUSY);
	}
	else {
		/* Re-init hal if we are being re-opened after a close. */
		if (zat->za_state == ZXE_HAL_RESET) {
			pdk_lock_init(ppa);
			pdk_mem_init(ppa);
			pdk_time_init(ppa);
			pdk_buf_init(ppa);
			pdk_layer_init(ppa);

			if (pdk_hal_init(ppa) != 0) {
				goto fail1;
			}

			zat->za_state = ZXE_HAL_INITTED;
			if (pdk_layer_push(ppa, PDK_HAL_LAYER) != 0) {
				DEBUG1("zxe_open(%d): pdk_layer_push failed\n", ppa);
				goto fail2;
			}

			if (pdk_layer_push(ppa, PDK_PAL_LAYER) != 0) {
				DEBUG1("zxe_open(%d): pdk_layer_push failed\n", ppa);
				goto fail2;
			}

		}
	}

#if ZNYX_NB || ZNYX_82559
#ifdef MODULE
	/* M11 hal only looks for the above five media names */
	for (i=0; i < 5; i++) {
		if (zmedia[ppa] && strcmp(zmedia[ppa], zxe_medianame[i].name) == 0) {
			DEBUG1("zxe_open(%d): setting znb media to %s\n",
				ppa, zxe_medianame[i].name);
			(*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_SET_MEDIATYPE, (void *) zxe_medianame[i].type);
			forcemedia = 1;
		} 
	}
	DEBUG1("zxe_debug_arg=%d zxe_debug=%d\n", zxe_debug_arg, zxe_debug);
	if (zxe_debug_arg > 1) zxe_debug = zxe_debug_arg;
#endif
#endif

	if (!forcemedia) {
		DEBUG1("zxe_open(%d): defaulting to media type auto.\n", ppa );
		(*zat->za_llr.pdk_ctl)(ppa, PDK_HAL_SET_MEDIATYPE, ZXE_MEDIA_AUTO);
	}

	if (znb_hwstart(ppa) != 0) {
		DEBUG1("zxe_open(%d): znb_hwstart failed\n", ppa);
		/* since hal_init() was moved, get rid of gotos */
		goto fail2;
	} else {
		DEBUG2("zxe_open(%d): znb_hwstart OK\n", ppa);
	}

	zat->za_flags = ZXE_OPEN;
#if (LINUX_VERSION_CODE >= 0x02032a)
	netif_start_queue(dev);
#else
	dev->tbusy = 0;
	dev->interrupt = 0;
	dev->start = 1;
/*	MOD_INC_USE_COUNT; */ /*moved to teh bottom (gh)*/
#endif
	MOD_INC_USE_COUNT; 
	dev->mtu = (*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_MTU_SIZE, (void *) 0);
	DEBUG3("zxe_open(%d): done\n", ppa);
	return(0);

fail2:
	DEBUG1("zxe_open(%d): at fail2\n", ppa);
	pdk_hal_deinit(ppa);
	zat->za_state = ZXE_HAL_RESET;

fail1:
	DEBUG1("zxe_open(%d): at fail1\n", ppa);
	if (znb_recently_initted != 0) {
		pdk_hal_global_deinit();
	}

	zxe_array[ppa] = 0;

	pdk_layer_deinit(ppa);
	pdk_time_deinit(ppa);
	pdk_buf_deinit(ppa);
	pdk_lock_deinit(ppa);
	pdk_mem_deinit(ppa);

	if (znb_recently_initted != 0) {
		pdk_layer_global_deinit();
		pdk_time_global_deinit();
		pdk_buf_global_deinit();
		pdk_lock_global_deinit();
		pdk_mem_global_deinit();
		znb_initted = 0;
	}
	return(-EAGAIN);
}

int
zxe_close(struct net_device *dev)
{
	zxe_array_t *zat;

	DEBUG1("zxe_close: dev 0x%x\n", dev);

	zat = ZXE_DEV_ARRAY(dev);
	if (zat == 0) {
		DEBUG3("zxe_close: no zat\n");
		return(-ENODEV);
	}

#if (LINUX_VERSION_CODE >= 0x02032a)
	netif_stop_queue(dev);
#else
	dev->start = 0;
	dev->tbusy = 1;
#endif

	/* Stop hardware */
	if (zat->za_state == ZXE_HAL_STARTED) {
		(*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_RECEIVE_DEINIT, 0);
		(*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_STOP_AND_RESET, 0);
		zat->za_state = ZXE_HAL_INITTED;
	}

	zat->za_flags &= ~ZXE_OPEN;

/*#if (LINUX_VERSION_CODE < 0x02032a)*/
	MOD_DEC_USE_COUNT;
/*#endif*/
	DEBUG3("zxe_close: done\n");

	return (0);
}

/* zxe_multi_req: called by Linux -- no mechanism for reporting errors. */

void
zxe_multi_req(struct net_device *dev)
{
	zxe_array_t *zat;
	struct dev_mc_list *mclist;
	int i;

	DEBUG3("zxe_multi_req: called\n");

	zat = ZXE_DEV_ARRAY(dev);
	if (zat == 0)
		return;

	if (dev->flags & IFF_PROMISC) {
		(*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_SET_PROMISCUOUS_PHYS, 0);
	}
	/* Tulip does hashing for anything between 14 and 1000. 
	 * The hal doesn't hash, and will fail above 14 -- but Linux
	 * doesn't seem to have any way to tell the kernel an address
	 * was rejected.
	 */
	else if (dev->mc_count > 14 || dev->flags & IFF_ALLMULTI) {
		(*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_SET_PROMISCUOUS_MULTI, 0);
	}
	else {
		/* We default to adding a multicast address.
		 * Linux doesn't have explicit ADD or DEL.
		 */
		mclist=dev->mc_list;
		for (i = 0; i < dev->mc_count; i++) {
			(*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_ADD_MULTICAST,
				(void *)mclist->dmi_addr);
			mclist = mclist->next;
		}
	}
}

/* linux/netdevice.h: (*set_mac_address)(struct net_device *dev, voide *addr);
 * Assume addr is a uchar addr[6]...so why is it void?
 */

int
zxe_set_physaddr_req(struct net_device *dev, void *addr)
{
	zxe_array_t *zat;
	pdk_ether_addr_t *addrp;
	
	struct sockaddr *p_sockaddr = (struct sockaddr *)addr;/*hamid*/
	addrp=(pdk_ether_addr_t *)p_sockaddr->sa_data;/*hamid*/;
	
	DEBUG3("zxe_set_physaddr_req: called\n");
        
	zat = ZXE_DEV_ARRAY(dev);
	if (zat == 0)
		return;

	
	if (((addrp->ether_addr_octet[0] & 1) == 1) || (ether_cmp(addrp, &etherbroadcastaddr) == 0)) {
		DEBUG1("zxe_set_physaddr_req(): trying to set physaddr as multicast or broadcast.\n"); 
		return;
	}

	/*
	 * Set new interface local address and re-init device.
	 * This is destructive to any other streams attached
	 * to this device.
	 */
	ether_copy(addrp, &zat->za_curaddr);

	(*zat->za_llr.pdk_ctl)(zat->za_ppa, PDK_HAL_SET_PHYSADDR, (void *)addrp);
        zxe_eaddr(zat->za_ppa)/*hamid*/;
	return;
}

/* Linux xmit() only fails if locked out, but doesn't free the skb... */

int
zxe_xmit(struct sk_buff *skb, struct net_device *dev)
{
	zxe_array_t *zat;
	unsigned long s;
	pdk_buf_t *bp;
	ulong err = 0;

	DEBUG3("zxe_xmit: called\n");

	s = splimp();
	zat = ZXE_DEV_ARRAY(dev);

#if (LINUX_VERSION_CODE >= 0x02032a)
	if (netif_queue_stopped(dev)) {
#else
	if (test_and_set_bit(0, (void *)&dev->tbusy) != 0) {
#endif
		splx(s);
		DEBUG2("zxe_xmit(%d): TX queue already stopped??\n", zat->za_ppa);
		return(1);
	}

#if (LINUX_VERSION_CODE >= 0x02032a)
        netif_stop_queue(dev);
#endif

	if ((bp = pdk_buf_alloc(zat->za_ppa, 0, skb)) == 0) {
#if 0
		if (zxe_debug > 2) freeqinfo(zat->za_ppa);
#endif
		splx(s);
		DEBUG2("zxe_xmit: buf_alloc fail\n");
		return(1);
	}

	bp->history=TXING;
	/* tulip doesn't even look for link up/down, it just fills the ring
	 * and triggers the xmit.  If down, what happens to that skb?
	 * Despite that -- 
	 * Since we have no wput(), let the skb get freed in pdk_buf_free().
	 */
	if ((err = (*zat->za_llr.pdk_xmit)(zat->za_ppa, bp)) != 0) {
		DEBUG2("zxe_xmit(%d): hal_xmit fail, bp %#x  skb %#x\n", zat->za_ppa, bp, bp->b_skb);
		
		if (err == PDK_LINK_DOWN)
			zat->za_nolink = 1;
		
		/* Linux requeues the skb */
		bp->b_skb = 0;
		pdk_buf_free(bp);
	}

	if (!zat->za_nolink) {
#if (LINUX_VERSION_CODE >= 0x02032a)
		netif_start_queue(dev);
#else
		clear_bit(0, (void *)&zat->za_dev->tbusy);
#endif
	}

	DEBUG3("zxe_xmit: done\n");
	splx(s);
	if (err)
		return(1);
	else
		return(0);
}

void
zxe_xmit_complete(ppa, bp, status)
int32_t ppa;
pdk_buf_t *bp;
uint32_t status;
{
	zxe_array_t *zat;

	DEBUG3("zxe_xmit_complete(%d): called\n", ppa);

	zat = zxe_array[ppa];
	if (zat == 0)
		return;

	bp->history=TXDONE;
	pdk_buf_free(bp);

	if (!zat->za_nolink) {
#if (LINUX_VERSION_CODE >= 0x02032a)
		netif_wake_queue(zat->za_dev);
#else
		clear_bit(0, (void *)&zat->za_dev->tbusy);
		mark_bh(NET_BH);
#endif
	}
}

void
zxe_recv_complete(ppa, bp, status)
int32_t ppa;
pdk_buf_t *bp;
uint32_t status;
{
	zxe_array_t *zat;
	struct net_device *dev;
	struct sk_buff *skb;
	static struct sk_buff* new_skb=NULL;
	ulong rx_num_bufs;
	static int has_fragments=0;
	struct pci_dev *pdev = znb_pdev[ppa];
	uint32_t pdk_etherrecv_size;

        /* added by Hamid*/
        if(status != 0 ){
                pdk_buf_free(bp);
		DEBUG2("zxe_recv_complete(%d): Received message with status 0x%x \n",ppa,status);
                return;
        }

	
#if 0
	if (pdev->device == SUBDEVICEID(ADAPTER_ZX231J))
		pdk_etherrecv_size = PDK_ETHERRECV_SIZE_MTU9K;
	else
#endif
		pdk_etherrecv_size = PDK_ETHERRECV_SIZE;

	zat = ZXE_PPA_ARRAY(ppa);
	if (zat == 0) {
		DEBUG1("zxe_recv_complete:  Received message on inactive ppa %d\n", ppa);
		pdk_buf_free(bp);
		/* fixme: added the return */
		return;
	}

	dev = ZXE_ARRAY_DEV(zat);

	bp->history=RXDONE;

	/* Peel off the skb */
	skb = bp->b_skb;
	bp->b_skb = 0;

#if LINUX_VERSION_CODE > 0x20127
	zxe_stats[ppa].rx_bytes += bp->b_len;
#endif

	(*zat->za_llr.pdk_ctl)(bp->b_ppa, PDK_HAL_GET_NUM_RECEIVE_BUFS, (void *) &rx_num_bufs);

	
	if (bp->b_len < RX_COPY_LEN || rx_num_bufs < RX_LOWATER) {
		DEBUG2("znbRX(%d): copy -- rx_num_bufs %d b_len %#x b_ppa %d\n",
		       ppa, rx_num_bufs, bp->b_len, bp->b_ppa);
		
		if ((new_skb = dev_alloc_skb(bp->b_len + 2)) != NULL) {
			if (zxe_debug > 1)
				pktdump(new_skb);
				
			new_skb->dev = dev;
			skb_reserve(new_skb, 2);  /* align the IP header */
			eth_copy_and_sum(new_skb, skb->data, bp->b_len, 0);
			skb_put(new_skb, bp->b_len);
			new_skb->protocol = eth_type_trans(new_skb, new_skb->dev);
			netif_rx(new_skb);
		} else {
			DEBUG2("znbRX(%d): copy fail, dropping pkt.\n");
		}
			
		/* restore and recycle the skb, since the new one went upstairs */
		bp->b_skb = skb;
		bp->b_len = pdk_etherrecv_size;
		(*zat->za_llr.pdk_recv)(bp->b_ppa, bp);
	} else {
		if ( (skb->tail + bp->b_len) > skb->end) {
			pdk_dumpskb(ppa, "OVER RX", bp, skb);
			pktdump(skb);
			bp->b_skb = skb;
			pdk_buf_free(bp);
			return;
		} else {
			if (zxe_debug > 2) pdk_dumpskb(ppa, "znbRX", bp, skb);
		}
		
		if (zxe_debug > 2)
			pktdump(skb);
		
		skb->dev = dev;
		skb_put(skb, bp->b_len);
		skb->protocol = eth_type_trans(skb, skb->dev);
		netif_rx(skb);
		pdk_buf_free(bp);		
	}
}

uint znb_save;

/* Assumes pt_regs is not used */

void
zxe_intr_handler(int ppa)
{
	zxe_array_t *zat = ZXE_PPA_ARRAY(ppa);

	zxe_intr(zat->za_dev->irq, zat->za_dev, (struct pt_regs *)0);
}


void
zxe_ntbridge_intr(int irq, void *dev_instance, struct pt_regs *regs)
{
	unsigned long s=splimp();
	struct net_device *dev = (struct net_device *) dev_instance;
	zxe_array_t *zat = ZXE_DEV_ARRAY(dev);

	pdk_ntbridge_intr(zat->za_ppa); 
	splx(s);
}

void
zxe_intr(int irq, void *dev_instance, struct pt_regs *regs)
{
	struct net_device *dev;
	unsigned long s;
	zxe_array_t *zat;
	ulong iac;

	s = splimp();
	dev = (struct net_device *) dev_instance;

	zat = ZXE_DEV_ARRAY(dev);

	if (zat == 0)
		return;

	/*
	spin_lock_irqsave(&znb_lock, znb_save);
	spin_lock(&znb_lock);
	*/


/* Is the buffer lock held?  If so, interrupts are *disabled*.
 * How did we get here?
 * check_bh_lkcnt(ppa);
 */

	DEBUG4("zxe_intr: Called on ppa %d\n", zat->za_ppa);

	/* Shouldn't be necessary with single processor.
	 * Add the processor_id check for SMP. 
	 */
	
	iac = (*zat->za_llr.pdk_iac)(zat->za_ppa);
	if (iac != 0) {
		(*zat->za_llr.pdk_intr)(zat->za_ppa, iac);
	}

#if (LINUX_VERSION_CODE < 0x02032a)
	clear_bit(0, (void *)&dev->interrupt);
#endif

/*
	spin_unlock(&znb_lock);
	spin_unlock_irqrestore(&znb_lock, znb_save);
*/
	splx(s);
}


/* Media Types */
char *mediatypes[] = {
	"ZXE_MEDIA_AUTO",
	"ZXE_MEDIA_TP",
	"ZXE_MEDIA_BNC",
	"ZXE_MEDIA_AUI",
	"ZXE_MEDIA_TPFD",
	"ZXE_MEDIA_TX",
	"ZXE_MEDIA_TXFD",
	"ZXE_MEDIA_T",
	"ZXE_MEDIA_AUTO_COMPLETE"  /* synonym ? "ZXE_MEDIA_MAX" */
	};


void
zxe_status(ppa, opcode, param)
int32_t ppa;
uint32_t opcode;
void *param;
{
	zxe_array_t *zat;
	ulong type;


	if (opcode == PDK_PSDB) {
		DEBUG2("zxe_status(%d): calling rmapi_notification\n", ppa);
		rmapi_notification((pdk_psdb_rhdr_t *) param);
		return;
	}

	zat = zxe_array[ppa];
	if (zat == 0)
		return;

	switch (opcode) {
		case PDK_STATUS_LINK_DOWN:
#if (LINUX_VERSION_CODE >= 0x02032a)
			netif_stop_queue(zat->za_dev);
#else
			set_bit(0, (void *)&zat->za_dev->tbusy);
#endif
			set_bit(0, (void *)&zat->za_nolink);
			DEBUG2("zxe_status(%d): LINK_DOWN  nolink %d\n", ppa, zat->za_nolink); 

			break;

		case PDK_STATUS_LINK_UP:
			(*zat->za_llr.pdk_ctl)(ppa, PDK_HAL_GET_MEDIATYPE, (void *)&type);
			DEBUG2("zxe_status(%d): HAL mediatype %s\n", ppa, mediatypes[type]);
			DEBUG2("zxe_status(%d): LINK_UP -- %s\n", ppa, mediatypes[type]); 
			clear_bit(0, (void *)&zat->za_nolink);
#if (LINUX_VERSION_CODE >= 0x02032a)
			netif_wake_queue(zat->za_dev);
#else
			clear_bit(0, (void *)&zat->za_dev->tbusy);
#endif
			break;

		case PDK_STATUS_LINK_CHANGE:
			(*zat->za_llr.pdk_ctl)(ppa, PDK_HAL_GET_MEDIATYPE, (void *)&type);
			DEBUG2("zxe_status(%d): LINK_CHANGE -- %s\n", ppa, mediatypes[type]); 
			break;

		case PDK_STATUS_INTERNAL_FAULT:
			DEBUG2("zxe_status(%d): INTERNAL FAULT\n", ppa); 
			break;
		case PDK_STATUS_EXTERNAL_FAULT:
			DEBUG2("zxe_status(%d): EXTERNAL FAULT\n", ppa); 
			break;
		default:
			DEBUG1("zxe_status(%d): Unsupported status: %d\n", ppa, opcode); 
			break;
	}
}

